# coding=utf-8
"""
library/parts.py - Auto-generated Part classes for the Python ldraw package.

Copyright (C) 2018 Matthieu Berthomé <rienafairefr@gmail.com>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


<< This file is auto-generated, you shouldn't have to modify it>>
"""
# pylint: disable=invalid-name, line-too-long, too-many-lines

AxeWithRedHeadAndSilverBladePattern = "95330p01"
BabyHead = "24581"
BabyHeadWithEyesAndSmilePattern = "24581p01"
BabyWithHeadWithEyesAndSmilePattern = "25126p01c01"
HeadAlienFemaleWithRedEyesAndDarkBluishGreyLipsAndWrinkles_OpenSmileWithMissingToothPattern = "3626cpce8"
HeadAlienSquidman = "85947"
HeadAlienSquidmanWithBlackEyes_LinesPattern = "85947p01"
HeadAlienWithLowerFangs_SingleOrangeEyeOpen_EyeHalfClosed2_SidedPattern = "3626bpc94"
HeadBlackEyeMaskWithWhiteEyeHolesAndDarkBrownCheekLinesPattern = "3626cpd8k"
HeadBlackEyebrows_Freckles_Smile_Worried2_SidedPattern = "3626cp8s"
HeadBlackEyebrows_WhitePupils_Scared_LopsideSmile2_SidedPattern = "3626cp8w"
HeadBlackHeadbandWithWhiteBatmanEyes_BroadSmileShowingTeethPattern = "3626cpd85"
HeadBlackStandardEyes_SmilingWithTongue_StandardGrin2_SidedPattern = "3626cp8e"
HeadBrightPinkHeadbandWithWhiteSquintedBatmanEyesPattern = "3626cpd63"
HeadBrownEyebrows_Freckles_OpenSmilePattern_HollowStudWithPiercedBase_ = "3626bp8b"
HeadBrownEyebrows_Freckles_OpenSmilePattern_HollowStud_ = "3626cp8b"
HeadCuboid = "19729"
HeadCuboidWithMinecraftPumpkinPattern = "19729p01"
HeadCuboidWithMinecraftStevePattern = "19729p02"
HeadElf = "43745"
HeadElfWithGreenEyesPattern = "43745p01"
HeadEwok = "64805"
HeadFemaleBlackEyebrows_Eyelashes_Freckles_PinkLips_LopsidedSmile_Determined2_SidedPattern = "3626cp8d"
HeadFemaleRobotWithRedEyesAndLips_4MetallicSilverMouthSquares_BlackBeautyMark_EyelashesAndHeadsetPattern = "3626cpd2b"
HeadFemaleWithBalaclava_MediumDarkFleshFace_BeautyMarkAndCrookedSmilePattern = "3626cpd89"
HeadFemaleWithBlackEyeMask_DarkRedLipsAndRedDiamondBeautyMark_WinkAndOpenMouth_LickingLips2_SidedPattern = "3626cpd81"
HeadFemaleWithBlackEyeMask_DarkRedLips_RedTongueAndDiamondOnCheek_OpenSmileWithTopTeethPattern = "3626cpd6d"
HeadFemaleWithBlackEyebrowsAndEyelashes_BeautyMarkUnderLeftEye_DarkRedLipsPattern = "3626cpd66"
HeadFemaleWithBlackEyebrowsAndEyesWithEyelash_BroadGrinPattern = "3626cpd8e"
HeadFemaleWithBlackEyebrows_BeautyMark_SingleEyelash_DarkTanLipsAndCrookedSmile_Scowl2_SidedPattern = "3626cpna"
HeadFemaleWithBlackEyebrows_PeachLips_OpenSmileWithTeeth_Sleeping2_SidedPattern = "3626cp8r"
HeadFemaleWithBlackEyebrows_PinkLips_Eyelashes_SunglassesWithMediumLavenderFrames2_SidedPattern = "3626cp8x"
HeadFemaleWithBlackEyebrows_WideEyelashesWithThickDarkAzureMascara_SmileAndDarkOrangeLipsPattern = "3626bpc5e"
HeadFemaleWithBlackEyes_ThinEyebrowsAndRedNarrowLips = "3626bpc42"
HeadFemaleWithBlackAndSilverSunglasses_BlackEyebrows_PeachLipsPattern_HollowStud = "3626cp8v"
HeadFemaleWithBrownEyebrows_MediumDarkFleshLipsAndBrownBeautyMarkOnLeftSidePattern = "3626cpd6h"
HeadFemaleWithBrownEyebrows_PeachLips_PensiveSmile_Scared2_SidedPattern = "3626cp8q"
HeadFemaleWithBrownEyebrows_Pupils_EyelashesPattern = "3626bp09"
HeadFemaleWithGlassesWithDarkGreenFrames_RedLips_Smiling_SmilingWithTeeth2_SidedPattern = "3626cpx2"
HeadFemaleWithGlassesWithGoldFrames_PeachLips_Smiling_SmilingWithTeeth2_SidedPattern = "3626cpx1"
HeadFemaleWithJaggedDarkAzureEyebrows_WhiteAtSidesOfEyes_SilverLipsPattern = "3626cpcg1"
HeadFemaleWithLargeRoundGlassesWithBrightLightBlueLensesAndBlackFrames_OrangeLipsWithSlightCrookedSmilePattern = "3626cpd7i"
HeadFemaleWithMagentaEyeMask_DarkRedLips_GrinWithTeeth_Neutral2_SidedPattern = "3626cpd6a"
HeadFemaleWithMediumDarkFleshGlasses_LaughLines_RedLips_Neutral_RaisedEyebrowsAmused2_SidedPattern = "3626cp8u"
HeadFemaleWithNose_RedLips_AndLargeEyesPattern = "3626bpw3"
HeadFemaleWithRedLipsAnnoyed_Smiling2_SidedPattern_HollowStudWithPiercedBase_ = "3626bp89"
HeadFemaleWithRedLipsAnnoyed_Smiling2_SidedPattern_HollowStud_ = "3626cp89"
HeadFemaleWithRedLipsSmallEyebrowsPattern = "3626bp08"
HeadFemaleWithSilverGlassesAndPinkLipsSmilingPattern = "3626cp8l"
HeadFemaleWithSmiling_Scared2_SidedPattern = "3626bpq3"
HeadFemaleWithThinBlackEyebrows_WhiteEyeShadow_OrangeLipsAndThreeBlackStripesOnCheeksPattern = "3626cpce9"
HeadFemaleWithWhiteSkiGogglesWithPinkAndPurpleGlassAndBlackStrapAndRedLipsPattern = "3626bpc87"
HeadGlasses_ThinBrownEyebrows_SmilePattern_HollowStudWithPiercedBase_ = "3626bp87"
HeadGlasses_ThinBrownEyebrows_SmilePattern_HollowStud_ = "3626cp87"
HeadGoblin = "42109"
HeadGrin_BrownBeard_WhitePupilsPattern_HollowStudWithPiercedBase_ = "3626bp88"
HeadGrin_BrownBeard_WhitePupilsPattern_HollowStud_ = "3626cp88"
HeadLeftyMouthAndStubblePattern = "3626bp81"
HeadMummyFaceWith2Eyes_1Eye2_SidedPattern = "3626bpq0"
HeadMummyWith2Eyes_GoldDeathMask2_SidedPattern = "3626bpq1"
HeadNautolan = "57901"
HeadNautolan_BlackEyes_Mouth_BrownTentacleBands = "57901p01"
HeadOpenSmileWithTeeth_Eyebrows_Scared2_SidedPattern = "3626cp8j"
HeadPeachLips_Smile_BlackEyebrowsPattern_HollowStudWithPiercedBase_ = "3626bp8c"
HeadPeachLips_Smile_BlackEyebrowsPattern_HollowStud_ = "3626cp8c"
HeadProtocolDroid = "30480"
HeadProtocolDroidWithLightYellowEyesPattern = "30480ps0"
HeadReddishBrownHeadbandWithSquintedBatmanEyes_SadMouthPattern = "3626cpd64"
HeadRobot = "98384"
HeadRobotWithBlueEyesAndRed_YellowTeethDisplay = "98384p01"
HeadRobotWithRedEyesAnd4BlackOutlinedSquaresMouthOnSilverBackgroundWithBlackRivetsPattern = "3626cp8p"
HeadRobotWithRedEyes_MetalEyebrowsWithRivets_BrokenWhiteTeethAndStubbleBoltsPattern = "3626cpd2e"
HeadRobotWithRedVisorAnd4BlackOutlinedSquaresMouthOnSilverBackgroundPattern = "3626cp8o"
HeadRobotWithRedVisorAnd9BlackSquaresMouthPattern = "3626cp8n"
HeadRodianWithEyesAndNosePattern = "47545p01"
HeadRodianWithoutPattern = "47545"
HeadSwJawa_YellowEyesWithOrangeRimPattern = "3626cps0"
HeadSimpsonsBartSimpson = "15523"
HeadSimpsonsBartSimpsonWithEyesLookingLeftPattern = "15523p01"
HeadSimpsonsBartSimpsonWithEyesPattern = "15523p02"
HeadSimpsonsHomerSimpson = "15527"
HeadSimpsonsHomerSimpsonWithHalfClosedEyesPattern = "15527p01"
HeadSimpsonsHomerSimpsonWithWideOpenEyesPattern = "15527p02"
HeadSimpsonsLisaSimpson = "15524"
HeadSimpsonsLisaSimpsonWithEyesPattern = "15524p02"
HeadSimpsonsLisaSimpsonWithWorriedEyesPattern = "15524p01"
HeadSimpsonsMargeSimpson = "15522"
HeadSimpsonsMargeSimpsonWithEyesLookingRightPattern = "15522p01"
HeadSimpsonsMargeSimpsonWithWideOpenEyesPattern = "15522p02"
HeadSimpsonsNedFlanders = "15529"
HeadSimpsonsNedFlandersWithGlasses_HairAndMoustachePattern = "15529p01"
HeadSkullCrackedWithMetalPlatesPattern = "3626cpn2"
HeadSpongebob = "54872"
HeadSpongebobWithHappyPattern = "54872p01"
HeadSpongebobWithShockedPattern = "54872p02"
HeadSpongebobWithSuperHeroPattern = "54872p09"
HeadSpongebobOctopus = "64767"
HeadSpongebobStarfish = "54873"
HeadTrandoshan = "87570"
HeadWhiteBushyEyebrows_Goatee_Wrinkles_Smile_OpenMouthScared2_SidedPattern = "3626cp8t"
HeadWookieeWithBandolier = "30483"
HeadWookieeWithSilverBandolier_BlackNosePattern = "30483p01"
HeadYodaWithCurvedEars = "41880"
HeadW_BigEyes_CurvedEyebrows_OrangeMouthPattern = "3626bpao"
HeadWithAdventurersMummyPattern = "3626bpa2"
HeadWithAngryEyebrowsAndTenseMouthPattern = "3626bp0f"
HeadWithAngryMoustacheAndTeethPattern = "3626bpw2"
HeadWithAngryMoustache_WhiteTeethAndGoldToothPattern = "3626bpw0"
HeadWithBalaclavaWithBlackEyeMaskWithWhiteEyeOpeningsAndLightFleshFaceAndScowlPattern = "3626cpbx"
HeadWithBalaclavaWithBlackEyesAndCurledEyelash_Z_InBlackCircle_LightFleshMouthHoleWithSneerPattern = "3626cpd6f"
HeadWithBalaclavaWithBlackSeamsAndWhiteEyesWithLightFleshLowerFacePattern = "3626cpd8c"
HeadWithBalaclavaWithWhiteFrowningEyesAndLightFleshMouthHole_ReddishBrownCheekLines_OpenSmileWithTeethPattern = "3626cpd6i"
HeadWithBeardStubble_Eyebrows_Smile_ScarPattern = "3626cpq0"
HeadWithBeard_Happy_Angry2_SidedPattern = "3626cpbb"
HeadWithBeard_Sneering_Scared2_SidedPattern = "3626cpbh"
HeadWithBlackBushyEyebrows_Smile_Angry2_SidedPattern = "3626cpnf"
HeadWithBlackEyebrowsAndCheekLines2_SidedPattern = "3626cpb9"
HeadWithBlackEyebrowsAndDarkBrownCheekLines_FirmExpressionPattern = "3626cpd8f"
HeadWithBlackEyebrows_BlackAndWhiteGoatee_WrinklesAndCheekLines_Neutral_Angry2_SidedPattern = "3626cpbm"
HeadWithBlackEyebrows_ChinDimpleAndLopsidedGrinPattern = "3626cp8i"
HeadWithBlackEyebrows_MediumDarkFleshCheekLines_DarkTanChinOrnamentAndSadLookPattern = "3626cpd6j"
HeadWithBlackEyebrows_ReddishBrownDimples_LopsidedOpenMouthGrinPattern = "3626cpd69"
HeadWithBlackEyebrows_WhitePupils_ChinDimple_OpenMouthSmileWithTeethPattern = "3626cp73"
HeadWithBlackFullBeard_ThickBlackEyebrowsAndWhitePupilsPattern = "3626bpq5"
HeadWithBlackHair_EyebrowsAndSmirkPattern = "3626bp0d"
HeadWithBlackMaskAndWhiteEyeHolesAndContourLinesPattern = "3626cpd6b"
HeadWithBlackMask_BlueEyesAndBlackLipsPattern = "3626bpba"
HeadWithBlackMoustache_BeardAndMessyHairPattern = "3626bp44"
HeadWithBlack_SilverSunglasses_BlackEyebrows_ChinDimpleAndGrimMouthPattern = "3626cp8z"
HeadWithBlueHeadbandAndDarkOrangeHairPattern = "3626bp66"
HeadWithBlueSunglassesPattern = "3626bp7b"
HeadWithBlueSunglassesAndStubblePattern = "3626bp7e"
HeadWithBlueWrap_AroundSunglassesPattern = "3626bp7c"
HeadWithBrightLightBlueGlassesWithBlackSquaredFrame_Smile_ScreamingOpenMouth2_SidedPattern = "3626cp8m"
HeadWithBrownAngularBeardAndGlassesPattern = "3626bp8d"
HeadWithBrownBushyMoustacheAndEyebrowsAndBlackChinStrapPattern = "3626cpcbe"
HeadWithBrownEyebrows_FleshCheekLines_Smile_SmileWithTeeth2_SidedPattern = "3626cpx5"
HeadWithBrownEyebrows_OpenLopsidedGrinAndWhitePupilsPattern_HollowStud_ = "3626cp8k"
HeadWithBrownEyebrows_SmirkingFace_PupilsPattern = "3626bp83"
HeadWithBrownEyebrows_WhitePupils_LopsidedSmileWithDimplePattern = "3626bp0g"
HeadWithBrownHairOverEyeAndBlackEyebrowsPattern = "3626bp7a"
HeadWithBrownHair_Eyelashes_AndLipstickPattern = "3626bpa6"
HeadWithBrownMoustache_Stubble_Eyebrows_FrowningPattern = "3626bp3r"
HeadWithClosedHollowStud = "3626c"
HeadWithDarkBluishGreyAroundEyesAndOpenMouthLopsidedPattern = "3626cpce7"
HeadWithDarkOrangeBeard_GoldTeethAndEyepatchPattern = "3626cp30"
HeadWithDarkOrangeMoustache_BeardAndMessyHairPattern = "3626bp42"
HeadWithDarkRedLips_Smirk_Eyemask2_SidedPattern = "3626bpb8"
HeadWithDarkredLips_Open_ClosedMouth_Freckles2_SidedPattern = "3626cpbj"
HeadWithEvilSkeletonSkullPattern = "3626bpa8"
HeadWithEyePatch_BlackStubbleAndMessyHairPattern = "3626bp49"
HeadWithEyePatch_DarkOrangeMoustache_BeardAndMessyHairPattern = "3626bp46"
HeadWithEyePatch_DarkOrangeStubbleAndMessyHairPattern = "3626bp48"
HeadWithEyePatch_RedMoustache_BeardAndMessyHairPattern = "3626bp47"
HeadWithEyebrows_Cheeklines_DeterminedPattern = "3626bpc6a"
HeadWithEyebrows_Crow_SFeet_OpenMouthSmile_QueasyExpressionWithSweatDrop2_SidedPattern = "3626cp70"
HeadWithEyeglassesAndLightningScarPattern = "3626bph1"
HeadWithEyesInMask_White_A_AndBlackLineOnBackPattern = "3626cpbl"
HeadWithFreckleNoseAndStandardGrinPattern = "3626bp07"
HeadWithFullBlackBeardWithSideBurns_WhitePupils_OpenMouthGrimacePattern = "3626cpx4"
HeadWithGlassesRounded_BrownEyebrowsAndSmilePattern = "3626cp8y"
HeadWithGlassesAndWhiteMuttonchopsPattern = "3626bpa1"
HeadWithGlasses_BrownSideburnsAndMoustachePattern = "3626bpq4"
HeadWithGlasses_GreyEyebrows_BeardAndMoustachePattern = "3626bp0e"
HeadWithGlasses_GreyEyes_Eyebrows_CheeksPattern = "3626cpbk"
HeadWithGlasses_PencilBehindEarAndPointedMoustachePattern = "3626bpw8"
HeadWithGoldPaintAndSlantedEyesPattern = "3626bp6u"
HeadWithGoldRobotPattern = "3626bp64"
HeadWithGreenBrainAndYellowMouthPattern = "3626bp6y"
HeadWithGreenEyebrows_MediumBlueEyeshadow_ThickRedLipsSmileWithSharpTooth_VerySadFrown2_SidedPattern = "3626cpd68"
HeadWithGreenLipsAndRedEyebrowsPattern = "3626bpb5"
HeadWithGregoryGoyle_HarryPotter2_SidedPattern = "3626bph4"
HeadWithGreyHair_Beard_Moustache_AngryPattern = "3626bp0a"
HeadWithGrin_DarkGreyBeardAndEyebrowsPattern = "3626bp39"
HeadWithGrumpy_Angry2_SidedPattern = "3626cpbd"
HeadWithHpHarryPotterWithGlassesAndLightningBolt_Smile_Frown2_SidedPattern = "3626bph7"
HeadWithHpNarcissaMalfoyWithDarkBrownEyebrows_Crow_SFeet_Angry_Smile2_SidedPattern = "3626bph9"
HeadWithHpRubeusHagridWithWrinklesPattern = "3626bph8"
HeadWithHpVoldemortWithTeethAndNostrilsPattern = "3626bph6"
HeadWithHalf_MoonGlassesAndGreyEyebrowsPattern = "3626bpha"
HeadWithHeadsetOverBrownHairAndEyebrowsPattern = "3626bp69"
HeadWithHollowStud = "3626b"
HeadWithIcePlanetFemaleRedHairPattern = "3626bp65"
HeadWithIcePlanetMessyWhiteHair = "3626bp62"
HeadWithIcePlanetMoustacheAndEyebrowsPattern = "3626bp61"
HeadWithInsectoidsRobotPattern = "3626bp50"
HeadWithIronManArmouredSuitClassicPattern = "3626cpb4"
HeadWithIronSpiderPattern = "3626cpb3"
HeadWithIslanderWhite_BluePaintedFacePattern = "3626bp3k"
HeadWithIslanderWhite_RedPaintedFacePattern = "3626bp3j"
HeadWithLotrDarkbrownBeard_StubbleAndStern_ClenchedTeeth2_SidedPattern = "3626cpm7"
HeadWithLotrFrowning_Grimacing2_SidedPattern = "3626cpm6"
HeadWithLotrFrowning_Scared2_SidedPattern = "3626cpm4"
HeadWithLotrFrowning_Shouting2_SidedPattern = "3626cpm5"
HeadWithLotrOrcWithGreyHairPattern = "3626bpma"
HeadWithLotrSmirking_Shouting2_SidedPattern = "3626cpm8"
HeadWithLotrThickGrayEyebrowsAndSmilePattern = "3626cpm1"
HeadWithLotrTired_Poisoned2_SidedPattern = "3626cpm3"
HeadWithLotrUruk_HaiLurtzScowlAndWhiteHand2_SidedPattern = "3626bpmb"
HeadWithLotrUruk_HaiScowlAndWhiteHand2_SidedPattern_HollowStudWithPiercedBase_ = "3626bpm0"
HeadWithLotrUruk_HaiScowlAndWhiteHand2_SidedPattern_HollowStud_ = "3626cpm0"
HeadWithLotrWorried_Smile2_SidedPattern = "3626cpm2"
HeadWithLargeBlackOpenMouthWithUvulaAndRedTonguePattern = "3626cpd6e"
HeadWithLargeBlueMaskPattern = "3626bp6v"
HeadWithLargeGlassesWithDarkGreyEyebrows_MoustacheAndStubblesAndDarkBrownCheekLinesPattern = "3626cpd67"
HeadWithLightTurquoiseAndOrangePaintedFacePattern = "3626bpw7"
HeadWithMaskBr_GreenWithEyeholesAndSmilePattern = "3626bpb2"
HeadWithMaskPurpleWithEyeholesAndSmilePattern = "3626bpb6"
HeadWithMediumAzureZigzagLinePattern = "3626cpcfb"
HeadWithMessyHairFemalePattern = "3626bp40"
HeadWithMessyHair_MoustacheAndStubblePattern = "3626bp35"
HeadWithMonocleAndBlackSlantedEyebrowsPattern = "3626bpb7"
HeadWithMonocle_Scar_AndMoustachePattern = "3626bpa7"
HeadWithMoustacheCurlyAndSpikyBeardUnderMouthPattern = "3626bp45"
HeadWithMoustache_StubbleAndBaldHairPartPattern = "3626bpw9"
HeadWithMouthClosedStern_MouthOpenScared2_SidedPattern = "3626cp80"
HeadWithOpenSmilingMouth_TeethAndToungePattern = "3626bpc44"
HeadWithOrangeBeardAndWhiteSmilePattern = "3626bp80"
HeadWithOrangePaintedFacePattern = "3626bpw4"
HeadWithPaintedTrianglesPattern = "3626bpw5"
HeadWithPursedLipsAndWhiteForeheadPattern = "3626bpb1"
HeadWithRaisedLeftEyebrow_FrecklesAndCrookedSmilePattern = "3626cp71"
HeadWithRedLipsAndBlackUpsweptEyelashesPattern = "3626bp6f"
HeadWithRedMoustacheAndGoateePattern = "3626bp41"
HeadWithRedMoustache_BeardAndMessyHairPattern = "3626bp43"
HeadWithRedAndDark_TurquoisePaintedLines_Eyebrows_Pupils_NosePattern = "3626bpw6"
HeadWithReddishBrownEyebrowsAndFreckles_LopsidedGrin_FrownPattern2_SidedPattern = "3626cpne"
HeadWithReddishBrownEyebrows_MediumDarkFleshScar_WhiteBand_Aid_LopsidedGrinWithTeeth_Determined2_SidedPattern = "3626cpnb"
HeadWithRonWeasleyPattern = "3626bph3"
HeadWithSwAlienWithLargeBlackEyesPattern = "3626bpsb"
HeadWithSwBlackEyebrowsAndScarsPattern = "3626bps7"
HeadWithSwBrownEyebrowsPattern = "3626bps2"
HeadWithSwBrownEyebrowsAndBeardPattern = "3626bps9"
HeadWithSwBrownEyebrows_SmileAndBlackChinStrapPattern = "3626cpsg"
HeadWithSwBrownEyebrows_SolidBlackEyesAndBlackChinStrapPattern = "3626cpsf"
HeadWithSwDarkBluishGreyChinStrap_SternEyebrows_BlackGoggles2_SidedPattern = "3626cpsi"
HeadWithSwDarthMaulPattern = "3626bps8"
HeadWithSwFrecklesAndThinBrownEyebrowsPattern = "3626bpsd"
HeadWithSwGreyBeardAndEyebrowsPattern = "3626bps4"
HeadWithSwGreyEyebrows_Implant2_SidedPattern = "3626bpsc"
HeadWithSwRedLips_BlackEyes_ThinEyebrowsPattern = "3626bps0"
HeadWithSwScoutTrooperBlackVisorPattern = "3626bpse"
HeadWithSwSmallBlackEyebrowsPattern = "3626bps3"
HeadWithSwSmirkAndBrownEyebrowsPattern = "3626bps5"
HeadWithSwSternBlackEyebrows_PupilsAndOrangeVisorPattern = "3626cpsh"
HeadWithSwTuskenRaiderPattern = "3626bpst"
HeadWithSeverusSnapePattern = "3626bphb"
HeadWithSideburnsAndDroopyMoustacheBlackPattern = "3626bp3n"
HeadWithSideburnsAndDroopyMoustacheBrownPattern = "3626bp3q"
HeadWithSideburns_Goatee_BeardStubble_ScarPattern = "3626bpq2"
HeadWithSilverMaskAndMouthGrillePattern = "3626bp6x"
HeadWithSilverRobotPattern = "3626bp63"
HeadWithSilverSunglassesAndRedHeadsetPattern = "3626bp68"
HeadWithSilverSunglasses_EyebrowsAndThinGrinPattern = "3626cp8h"
HeadWithSingleLargeBlackEyeWithPupilPattern = "3626cp02"
HeadWithSkullType1_Smiling_Pattern = "3626bpaq"
HeadWithSmallBlueMaskPattern = "3626bp6w"
HeadWithSmileAndGoldToothPattern = "3626bp86"
HeadWithSmile_BlackEyebrowsAndWhitePupilsPattern = "3626bp84"
HeadWithSmile_BrownEyebrowsAndWhitePupilsPattern = "3626bp85"
HeadWithSmirk_BlackMoustachePattern = "3626bpa3"
HeadWithSmirk_BlackHairAndGoateePattern = "3626bp82"
HeadWithSmirk_BlackMoustacheAndCleftChinPattern = "3626bpap"
HeadWithSolidStud = "3626a"
HeadWithStandardGrinPattern_HollowStudWithPiercedBase_ = "3626bp01"
HeadWithStandardGrinPattern_HollowStud_ = "3626cp01"
HeadWithStandardGrinPattern_SolidStud_ = "3626ap01"
HeadWithStandardGrinAndEyebrowsPattern = "3626bp05"
HeadWithStandardGrinAndPointedMoustachePattern = "3626bp03"
HeadWithStandardGrinAndSunglassesPattern = "3626bp04"
HeadWithStandardGrin_EyebrowsAndMicrophonePattern = "3626bp06"
HeadWithStandardWomanPattern = "3626bp02"
HeadWithStubble_MoustacheAndSmirkPattern = "3626bpa5"
HeadWithTanEyebrowsAndFrownPattern = "3626bph2"
HeadWithTeeth_Pupils_BrownEyebrowsPattern = "3626bp0b"
HeadWithThickAngryMoustache_StubbleAndLongHairPattern = "3626bpw1"
HeadWithThickBlackEyebrows_LavenderBlackEye_AdhesiveBandageAndFrownPattern = "3626cpcf4"
HeadWithThickBlackEyebrows_RedEyesWithWhitePupils_LightBluishGreyEyeShadowAndFangsPattern = "3626bpc25"
HeadWithThickBlackMoustacheAndEyebrows_GrinWithTeeth = "3626bpc21"
HeadWithThickMoustacheAndShortEyebrowsPattern = "3626bp0c"
HeadWithThinMoustache_StubbleAndSideburnsPattern = "3626bp67"
HeadWithTribalPaintAndFrownPattern = "3626bpac"
HeadWithVillainBlackFacialHairPattern = "3626bpa4"
HeadWithVillainousGlasses_BlackFacialHairPattern = "3626bpa9"
HeadWithVincentCrabbe_RonWeasley2_SidedPattern = "3626bph5"
HeadWithWhiteHair_Eyebrows_AndMoustachePattern = "3626bpn1"
HeadWithWhiteHeadbandAndFleshCheekLines_SmirkPattern = "3626cpd61"
HeadWithWhiteSkullMask_YellowEyesAndWhiteStringTiedInBackPattern = "3626cpceb"
HeadWithWhiteAndBlueStripesOnCheeks_BlackEyebrowsAndBigGrinPattern = "3626bpc33"
HeadWithWhiteAndLightBluishGreyBushyEyebrows_OpenMouthSmilePattern = "3626bp0h"
HeadWithWhiteAndLightBluishGreyEyebrows_MoustachAndGoatee_DarkOrangeWrinkles_ConcernedExpressionPattern = "3626cpnd"
HeadWithWideSmile_RedLipsAndCrow_SFeetPattern = "3626bpb3"
HeadWithWiryMoustache_GoateeAndEyebrowsPattern = "3626bp3e"
HeadWithYellowEyesAndMouthAndAluminumFoilBlotchesPattern = "3626cp8f"
HelmetWithWingsAndEagleHead = "24088"
MartianHeadWithClip = "30529"
MartianHeadWithClipWithEyelashesPattern = "30529p02"
MartianHeadWithClipWithEyesPlainPattern = "30529p01"
MechanicalHeadSwBattleDroid = "30378"
MechanicalHeadSwBattleDroidWithDarkRedInsigniaPattern = "30378ps2"
MechanicalHeadSwBattleDroidWithOrangeInsigniaPattern = "30378ps1"
MechanicalHeadSwBattleDroidWithYellowInsigniaPattern = "30378ps3"
Pike_SpearElaborateWithFlatSilverHead = "90391p02"
Pike_SpearElaborateWithMetallicSilverHead = "90391p01"
ShieldOvoidWithBullHeadPattern = "2586p4g"
ShieldOvoidWithBullHeadOnBrownBorderSticker = "2586d01"
ShieldTriangularLongWithBearHeadOnDarkRedAndRedBackgroundPattern = "18836p40"
ToolScrewdriverWithWideHeadAnd3_RibHandle = "604550"



"""
takes care of reading and writing a configuration in config.yml
"""
import os
from os.path import join

import sys
import yaml

from ldraw.dirs import get_config_dir, get_data_dir


def get_config_file_path():
    """ get the config file path """
    return join(get_config_dir(), 'config.yml')


def get_config():
    """ get the configuration from config.yml, create it if not there """
    data_dir = get_data_dir()
    try:
        config = yaml.load(open(get_config_file_path(), 'r'))
        return config
    except (OSError, yaml.YAMLError, IOError, EnvironmentError) as e:
        parts_lst_path = os.environ.get('LDRAW_PARTS_LST')
        if parts_lst_path:
            return {
                'parts.lst': parts_lst_path
            }
        else:
            return {
                'parts.lst': join(data_dir, 'ldraw', 'parts.lst')
            }


def write_config(config_dict):
    """ write the config to config.yml """
    yaml.dump(config_dict, open(get_config_file_path(), 'w'))


if __name__ == '__main__':
    print('Configuration file path:')
    print(get_config_file_path())
    print('Configuration used:')
    yaml.dump(get_config(), sys.stdout, default_flow_style=False)


"""
__init__.py - Package file for the ldraw Python package.

Copyright (C) 2008 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import print_function
import sys
import imp

import os

from ldraw.config import get_config, write_config
from ldraw.library_gen import library_gen_main
from ldraw.dirs import get_data_dir, get_config_dir
from ldraw.download import download_main


from pkg_resources import get_distribution, DistributionNotFound
try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # package is not installed
    pass


def load_lib_from(fullname, library_dir):
    """ load library from a dir """
    dot_split = fullname.split('.')
    dot_split.pop(0)
    lib_name = dot_split[-1]
    lib_dir = os.path.join(library_dir, *tuple(dot_split[:-1]))
    info = imp.find_module(lib_name, [lib_dir])
    library_module = imp.load_module(lib_name, *info)
    return library_module


def load_lib(library_path, fullname):
    """ try to load from library """
    return load_lib_from(fullname, library_path)


def try_download_generate_lib():
    # Download the library and generate it, if needed
    config = get_config()
    parts_lst_path = config['parts.lst']
    download_main(parts_lst_path)
    data_dir = get_data_dir()
    library_path = config.get('library')
    if library_path is not None:
        library_gen_main(parts_lst_path, library_path)
        return library_path
    else:
        try:
            # try to write the library to ldraw package folder (can work if user-writeable)
            ldraw_path = os.path.abspath(os.path.dirname(__file__))
            library_gen_main(parts_lst_path, ldraw_path)
            return ldraw_path
        except (OSError, IOError):
            # Failed, then write it to data_dir
            library_gen_main(parts_lst_path, data_dir)
            return data_dir


class CustomImporter(object):
    """ Added to sys.meta_path as an import hook """
    virtual_module = 'ldraw.library'

    @classmethod
    def valid_module(cls, fullname):
        if fullname.startswith(cls.virtual_module):
            rest = fullname[len(cls.virtual_module):]
            if not rest or rest.startswith('.'):
                return True

    def find_module(self, fullname, path=None): # pylint:disable=unused-argument
        """
        This method is called by Python if this class
        is on sys.path. fullname is the fully-qualified
        name of the module to look for, and path is either
        __path__ (for submodules and subpackages) or None (for
        a top-level module/package).

        Called every time an import
        statement is detected (or __import__ is called), before
        Python's built-in package/module-finding code kicks in.
        """
        if self.valid_module(fullname):
            # As per PEP #302 (which implemented the sys.meta_path protocol),
            # if fullname is the name of a module/package that we want to
            # report as found, then we need to return a loader object.
            # In this simple example, that will just be self.

            return self

        # If we don't provide the requested module, return None, as per
        # PEP #302.

        return None

    @classmethod
    def clean(cls):
        for fullname in list(sys.modules.keys()):
            if cls.valid_module(fullname):
                del sys.modules[fullname]

    def get_code(self, fullname):
        return None

    def load_module(self, fullname):
        """
        This method is called by Python if CustomImporter.find_module
        does not return None. fullname is the fully-qualified name
        of the module/package that was requested.
        """

        if not self.valid_module(fullname):
            # Raise ImportError as per PEP #302 if the requested module/package
            # couldn't be loaded. This should never be reached in this
            # simple example, but it's included here for completeness. :)
            raise ImportError(fullname)

        # if the library already exists and correctly generated,
        # the __hash__ will prevent re-generation
        generated_library_path = try_download_generate_lib()
        mod = load_lib(generated_library_path, fullname)
        sys.modules[fullname] = mod
        return mod


if not any(isinstance(o, CustomImporter) for o in sys.meta_path):
    # Add our import hook to sys.meta_path
    sys.meta_path.insert(0, CustomImporter())



"""
povray.py - A POV-Ray writer for the ldraw Python package.

Copyright (C) 2009 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys

from ldraw.library.colours import Main_Colour as Current
from ldraw.geometry import Vector
from ldraw.lines import Quadrilateral, Triangle
from ldraw.pieces import Piece

POV_OBJECT2 = """object {
%s
"""

POV_OBJECT = """  object {
    %s
"""

POV_DECLARE = """#declare %%s = %s {
"""

POV_DECLARE_OBJECT = POV_DECLARE % 'object'
POV_DECLARE_UNION = POV_DECLARE % 'union'

POV_PREAMBLE = POV_DECLARE_UNION % 'scene'

POV_LIGHTSOURCE = """light_source {
  <%1.3f, %1.3f, %1.3f>, %s
}

"""

POV_POSTAMBLE = """}
object {
  scene
  matrix <1.0, 0.0, 0.0,
          0.0, -1.0, 0.0,
          0.0, 0.0, 1.0,
          0.0, 0.0, 0.0>
}
"""

POV_FORMAT_STRING_TRIANGLE = "  <%1.3f, %1.3f, %1.3f>, " \
                             "<%1.3f, %1.3f, %1.3f>, " \
                             "<%1.3f, %1.3f, %1.3f>\n"

POV_FORMAT_STRING = """matrix <%1.3f, %1.3f, %1.3f,
%1.3f, %1.3f, %1.3f,
%1.3f, %1.3f, %1.3f,
%1.3f, %1.3f, %1.3f>"""


def _rgb_line(alpha, red, green, blue):
    rgb = (red, green, blue)
    if alpha:
        return "rgbt <%1.1f, %1.1f, %1.1f, %1.1f>" % (rgb + (alpha,))
    return "rgb <%1.1f, %1.1f, %1.1f>" % rgb


def _object_name(part):
    # Replace forbidden characters to produce a valid object name.
    return "part" + part.replace("-", "_").replace("\\", "_").replace("#", "_")


class POVRayWriter(object):
    # pylint: disable=too-few-public-methods
    """
        Writes a POV file from a model
    """
    ColourAttributes = {
        "CHROME": ("metallic 1.0", "specular 0.8", "brilliance 3", "diffuse 0.6"),
        "PEARLESCENT": ("diffuse 0.7", "specular 0.8"),
        "RUBBER": ("diffuse 1.0",),
        "MATTE_METALLIC": ("metallic 0.5", "roughness 0.2"),
        "METAL": ("metallic 0.8", "specular 0.8", "reflection 0.5")
    }

    def __init__(self, parts, pov_file):
        self.parts = parts
        self.pov_file = pov_file
        self.lights = []
        self.minimum = Vector(0, 0, 0)
        self.maximum = Vector(0, 0, 0)
        self.warnings = []

    def write(self, model):
        """
        Writes the provided model to self.pov_file
        :param model: a Part from a .ldr file
        :return:
        """

        # Define objects for the pieces first.
        objects = {}
        ordered_objects = []
        for obj in model.objects:
            if isinstance(obj, Piece):
                ordered_objects += self._create_piece_objects(obj, objects)
        # Write the objects, discarding any that are invalid in any way.
        for part in ordered_objects:
            if not self._write_object_definition(part, objects):
                del objects[part]
        # Write the pieces using the objects.
        self.pov_file.write(POV_PREAMBLE)
        for obj in model.objects:
            if isinstance(obj, Piece):
                self._write_piece(obj, objects)
            elif isinstance(obj, Triangle):
                self._write_triangle_1(obj)
            elif isinstance(obj, Quadrilateral):
                self._write_triangle_1(obj)
                self._write_triangle_2(obj)
        self.pov_file.write(POV_POSTAMBLE)

    def _write_triangle_2(self, obj):
        if abs((obj.point3 - obj.point1).cross(obj.point4 - obj.point1)) != 0:
            self._write_triangle(obj.colour, obj.point3, obj.point4, obj.point1)

    def _write_triangle_1(self, obj):
        if abs((obj.point3 - obj.point1).cross(obj.point2 - obj.point1)) != 0:
            self._write_triangle(obj.colour, obj.point1, obj.point2, obj.point3)

    def _write_piece(self, obj, objects):
        if obj.part in objects:
            obj_position = (obj.position.x, obj.position.y, obj.position.z)
            self.pov_file.write(POV_OBJECT2 % _object_name(obj.part))
            self._write_colour(obj.colour, 2)
            transposed_matrix = obj.matrix.transpose().flatten()
            pov_line = transposed_matrix + obj_position
            self.pov_file.write(POV_FORMAT_STRING % pov_line)
            self.pov_file.write("}\n\n")
        elif obj.part == "LIGHT":
            obj_position = (obj.position.x, obj.position.y, obj.position.z)
            pov_line = obj_position + (self._colour_string(obj.colour),)
            self.pov_file.write(POV_LIGHTSOURCE % pov_line)
            self.lights.append(obj)

    def _write_piece_2(self, obj):
        # Refer to the object for the other piece.
        self.pov_file.write(POV_OBJECT % _object_name(obj.part))
        # Apply a transformation for this particular piece.
        transposed_matrix = obj.matrix.transpose().flatten()
        obj_position = (obj.position.x, obj.position.y, obj.position.z)
        pov_line = transposed_matrix + obj_position
        self.pov_file.write(POV_FORMAT_STRING % pov_line)
        self._write_colour(obj.colour, 2)
        self.pov_file.write("  }\n")

    def _rgb_from_colour(self, colour):
        colour = self.parts.colours.get(colour.code, "#FFFFFF")
        red = int(colour[1:3], 16) / 255.0
        green = int(colour[3:5], 16) / 255.0
        blue = int(colour[5:7], 16) / 255.0
        return red, green, blue

    def _alpha_from_colour(self, colour):
        return self.parts.alpha_values.get(colour.code, 0) / 255.0

    def _colour_string(self, colour):
        red, green, blue = self._rgb_from_colour(colour)
        alpha = self._alpha_from_colour(colour)
        return _rgb_line(alpha, red, green, blue)

    def _finish_string(self, colour):
        attributes = self.parts.colour_attributes.get(colour.code, [])
        if attributes:
            attributes = POVRayWriter.ColourAttributes.get(attributes[0], [])
            return "\n    ".join(attributes)
        return ""

    def _write_triangle(self, colour, *vectors):
        indent = 2
        assert len(vectors) == 3
        self.minimum = Vector(min(self.minimum.x, vectors[0].x, vectors[1].x, vectors[2].x),
                              min(self.minimum.y, -vectors[0].y, -vectors[1].y, -vectors[2].y),
                              min(self.minimum.z, vectors[0].z, vectors[1].z, vectors[2].z))
        self.maximum = Vector(max(self.maximum.x, vectors[0].x, vectors[1].x, vectors[2].x),
                              max(self.maximum.y, -vectors[0].y, -vectors[1].y, -vectors[2].y),
                              max(self.maximum.z, vectors[0].z, vectors[1].z, vectors[2].z))
        triangle = (
            vectors[0].x, vectors[0].y, vectors[0].z,
            vectors[1].x, vectors[1].y, vectors[1].z,
            vectors[2].x, vectors[2].y, vectors[2].z
        )
        lines = ["triangle", "{", POV_FORMAT_STRING_TRIANGLE % triangle]
        self.pov_file.write("\n".join(indent * " " + x for x in lines))
        self._write_colour(colour, indent + 2)
        self.pov_file.write(" " * indent + "}\n\n")

    def _write_object_definition(self, part, objects):
        definition = objects[part]
        # Examine the constituent objects, keeping only those that refer to
        # known pieces and that are described using non-singular matrices.
        allowed = []
        for obj in definition:
            if not isinstance(obj, Piece):
                allowed.append(obj)
                continue
            if obj.part not in objects:
                self.warnings.append(("Discarding reference to %s", obj.part))
                continue
            if obj.matrix.fix_diagonal():
                self.warnings.append(("Correcting diagonal matrix elements for %s.", obj.part))
            if obj.matrix.det() == 0.0:
                self.warnings.append(("Discarding %s with singular matrix.", obj.part))
                continue
            allowed.append(obj)
        if not allowed:
            return False
        self.pov_file.write("// Part %s\n\n" % part)
        if len(definition) > 1:
            self.pov_file.write(POV_DECLARE_UNION % _object_name(part))
        else:
            self.pov_file.write(POV_DECLARE_OBJECT % _object_name(part))
        for obj in allowed:
            if isinstance(obj, Piece):
                self._write_piece_2(obj)
            elif isinstance(obj, Triangle):
                if abs((obj.point3 - obj.point1).cross(obj.point2 - obj.point1)) != 0:
                    self._write_triangle(obj.colour, obj.point1, obj.point2, obj.point3)
        self.pov_file.write("}\n\n")
        return True

    def _write_colour(self, colour, indent):
        if colour == Current:
            return
        self.pov_file.write(indent * " ")
        self.pov_file.write("pigment { color %s }\n" % self._colour_string(colour))
        finish = self._finish_string(colour)
        if finish.strip():
            self.pov_file.write(indent * " ")
            self.pov_file.write("finish { %s }\n" % finish)

    def _create_piece_objects(self, this_obj, objects):
        if this_obj.part in objects:
            return []
        part = self.parts.part(code=this_obj.part)
        if not part:
            sys.stderr.write("Part not found: %s\n" % this_obj.part)
            return []
        definition = []
        ordered_objects = []
        for obj in part.objects:
            if isinstance(obj, Piece):
                # Define this piece, too.
                ordered_objects = ordered_objects + self._create_piece_objects(obj, objects)
                # Record the object as part of the piece's definition.
                definition.append(obj)
            elif isinstance(obj, Triangle):
                # Record the object as part of the piece's definition.
                definition.append(obj)
            elif isinstance(obj, Quadrilateral):
                # Split the quadrilateral into two triangles.
                triangle1 = Triangle(obj.colour, obj.point1, obj.point2, obj.point3)
                triangle2 = Triangle(obj.colour, obj.point3, obj.point4, obj.point1)
                definition.append(triangle1)
                definition.append(triangle2)
        if definition:
            objects[this_obj.part] = definition
            ordered_objects.append(this_obj.part)
        return ordered_objects



# coding=utf-8
"""
library/parts.py - Auto-generated Part classes for the Python ldraw package.

Copyright (C) 2018 Matthieu Berthomé <rienafairefr@gmail.com>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


<< This file is auto-generated, you shouldn't have to modify it>>
"""

__all__ = [
    'accessories',
    'hats',
    'heads',
    'torsos',
    'legs',
    'arms',
    'hands',
]



""" compatibility module for python 2/3 """
import sys

PY2 = sys.version_info[0] == 2

if PY2:
    from urllib import urlretrieve
    from StringIO import StringIO
    from __builtin__ import reduce

    do_execfile = execfile
else:
    from io import StringIO
    from urllib.request import urlretrieve
    from functools import reduce

    def do_execfile(script, global_vars, local_vars):
        with open(script) as f:
            code = compile(f.read(), script, 'exec')
            exec(code, global_vars, local_vars)




"""
Common code for all the Writers
"""
import sys

from ldraw.geometry import Identity, Vector
from ldraw.library.colours import White, Main_Colour
from ldraw.lines import Triangle, Quadrilateral, Line
from ldraw.pieces import Piece


def _current_colour(colour, current_colour):
    return current_colour if colour == Main_Colour else colour.code


class Current(object):
    """ an instance of this is passed around during rendering """
    # pylint: disable=too-few-public-methods

    def __init__(self, matrix, colour, position):
        self.matrix = matrix
        self.colour = colour
        self.position = position


class Writer(object):
    # pylint: disable=too-many-arguments, too-few-public-methods
    """ Common logic for PNG, SVG, POV writers """

    def __init__(self, camera_position, system, parts):
        self.camera_position = camera_position
        self.system = system
        self.parts = parts

    def _opacity_from_colour(self, colour):
        return self.parts.alpha_values.get(colour, 255) / 255.0

    def _polygons_from_objects(self,
                               model,
                               top_level_piece=None,
                               current=Current(Identity(), White.code, Vector(0, 0, 0))):
        # Extract polygons from objects, filtering out those behind the camera.
        polygons = []

        poly_handlers = {
            Piece: self._subpart_get_poly,
            Line: self._line_get_poly,
            Triangle: self._triangle_get_poly,
            Quadrilateral: self._quadrilateral_get_poly,
        }

        for obj in model.objects:
            if isinstance(obj, Piece) and obj.part == "LIGHT":
                continue
            try:
                args = (obj,
                        top_level_piece or obj,
                        current)
                poly = poly_handlers[type(obj)](*args)
            except KeyError:
                continue
            if poly:
                polygons.extend(poly)
            else:
                continue

        return polygons

    def _line_get_poly(self,
                       obj,
                       top_level_piece,
                       current):
        pass

    def _subpart_get_poly(self,
                          obj,
                          top_level_piece,
                          current):
        colour = _current_colour(obj.colour, current.colour)
        part = self.parts.part(code=obj.part)
        if part:
            matrix = obj.matrix
            new_current = Current(current.matrix * matrix,
                                  colour,
                                  current.position + current.matrix * obj.position)
            return self._polygons_from_objects(part,
                                               top_level_piece,
                                               new_current)
        sys.stderr.write("Part not found: %s\n" % obj.part)
        return False

    def _triangle_get_poly(self,
                           obj,
                           top_level_piece,
                           current):
        camera_position = self.camera_position

        points = [current.matrix * p + current.position - camera_position for p in obj.points]
        if abs((points[2] - points[0]).cross(points[1] - points[0])) == 0:
            return False

        return self._common_get_poly(obj, top_level_piece, current.colour, points)

    def _common_get_poly(self,
                         obj,
                         top_level_piece,
                         current_colour,
                         points):
        projections = [self.system.project(p) for p in points]
        if any(p.z >= 0 for p in projections):
            return False
        colour = _current_colour(obj.colour, current_colour)

        return self._get_polygon(top_level_piece, colour, projections)

    def _quadrilateral_get_poly(self,
                                obj,
                                top_level_piece,
                                current):
        camera_position = self.camera_position

        points = [current.matrix * p + current.position - camera_position for p in obj.points]

        if abs((points[2] - points[0]).cross(points[1] - points[0])) == 0:
            return False
        if abs((points[2] - points[0]).cross(points[3] - points[0])) == 0:
            return False

        return self._common_get_poly(obj, top_level_piece, current.colour, points)

    def _get_polygon(self, top_level_piece, colour, projections):
        pass



#!/usr/bin/env python

"""
ldr2png.py - An LDraw to PNG convertor tool.

Copyright (C) 2010 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import argparse

from PIL import ImageColor

from ldraw.tools import (widthxheight, vector_position, get_model,
                         get_coordinate_system, verify_camera_look_at)
from ldraw.writers.png import PNGWriter, PNGArgs


def main():
    """ ldr2png main function """
    description = """Converts the LDraw file to a PNG file.
    
The image size must be specified in the format <width>x<height> where the width
and height are measured in pixels.

The camera and look-at positions are x,y,z arguments in LDraw scene coordinates
where each coordinate should be specified as a floating point number.

The eye to viewport distance is specified as a floating point number representing
a length in LDraw scene coordinates. Its default value is 1.0.

The optional sky background and stroke colours are PNG colours, either specified as
#rrggbb or as a named colour.

"""

    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('ldraw_file')
    parser.add_argument('png_file')
    parser.add_argument('image_size', type=widthxheight)
    parser.add_argument('camera_position', type=vector_position)
    parser.add_argument('look_at_position', required=False, default=vector_position('0,0,0'))
    parser.add_argument('--distance', type=float, default=1.0)
    parser.add_argument('--stroke-colour', dest='stroke_colour', type=ImageColor.getrgb)
    parser.add_argument('--sky', default=ImageColor.getrgb('#000000'), type=ImageColor.getrgb)

    args = parser.parse_args()

    png_args = PNGArgs(args.distance, args.image_size, args.stroke_colour, args.sky)

    ldr2png(args.ldraw_file, args.png_file, args.look_at_position, args.camera_position, png_args)


def ldr2png(ldraw_path, png_path, look_at_position, camera_position, png_args):
    """ Implementation of ldr2png """
    verify_camera_look_at(camera_position, look_at_position)

    model, parts = get_model(ldraw_path)

    system = get_coordinate_system(camera_position, look_at_position)

    writer = PNGWriter(camera_position, system, parts)
    writer.write(model, png_path, png_args)


if __name__ == "__main__":
    main()



"""
parts.py - Part management classes for the Python ldraw package.

Copyright (C) 2008 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
# pylint: disable=too-few-public-methods

import os
import re
import codecs


import inflect
from attrdict import AttrDict

from ldraw.colour import Colour
from ldraw.config import get_config
from ldraw.geometry import Matrix, Vector
from ldraw.lines import OptionalLine, Quadrilateral, Line, Triangle, MetaCommand, Comment
from ldraw.pieces import Piece
from ldraw.utils import camel


class PartError(Exception):
    """ An exception happening during Part file processing """
    pass


DOT_DAT = re.compile(r"\.DAT", flags=re.IGNORECASE)
ENDS_DOT_DAT = re.compile(r"\.DAT$", flags=re.IGNORECASE)


class Parts(object):
    # pylint: disable=too-many-instance-attributes
    """ Part class """
    ColourAttributes = ("CHROME", "PEARLESCENT", "RUBBER", "MATTE_METALLIC",
                        "METAL")

    def __init__(self, path=get_config()['parts.lst']):
        self.path = None
        self.parts_dirs = []
        self.parts_subdirs = {}
        self.parts_by_name = {}
        self.parts_by_code = {}

        self.primitives_by_name = {}
        self.primitives_by_code = {}

        self.colours = {}
        self.alpha_values = {}
        self.colour_attributes = {}

        self.colours_by_name = {}
        self.colours_by_code = {}

        self.parts = {
            'minifig': {
                'hats': {},
                'heads': {},
                'torsos': {},
                'hips': {},
                'legs': {},
                'arms': {},
                'hands': {},
                'accessories': {},
            },
            'others': {

            }}
        inflect_engine = inflect.engine()

        self.minifig_descriptions = {
            k: camel(inflect_engine.singular_noun(k)) for k in self.parts['minifig']
        }

        self.load(path)

    def load(self, path):
        """ load a Part from a path """
        try:
            self.try_load(path)
        except IOError:
            raise PartError("Failed to load parts file: %s" % path)

        # If we successfully loaded the files then record the path and look for
        # part files.
        self.path = path
        directory = os.path.split(self.path)[0]
        for item in os.listdir(directory):
            obj = os.path.join(directory, item)
            if item.lower() == "parts" and os.path.isdir(obj):
                self.parts_dirs.append(obj)
                self._find_parts_subdirs(obj)
            elif item.lower() == "p" and os.path.isdir(obj):
                self.parts_dirs.append(obj)
                self._find_parts_subdirs(obj)
            elif item.lower() == "ldconfig" + os.extsep + "ldr":
                self._load_colours(obj)
            elif item.lower() == "p" + os.extsep + "lst" and os.path.isfile(obj):
                self._load_primitives(obj)

        self.parts['minifig'] = AttrDict(self.parts['minifig'])
        self.parts = AttrDict(self.parts)

    def try_load(self, path):
        """ try loading a Part from a path """
        part_file = codecs.open(path, 'r', encoding='utf-8')
        for line in part_file.readlines():
            pieces = re.split(DOT_DAT, line)
            if len(pieces) != 2:
                break

            code, description = self.section_find(pieces)
            self.parts_by_name[description] = code
            self.parts_by_code[code] = description

    def section_find(self, pieces):
        """ returns code, description from a pieces element """
        code = pieces[0]
        description = pieces[1].strip()
        for key, section in self.parts['minifig'].items():
            searched = self.minifig_descriptions[key]
            index_find = description.find(searched)

            if index_find != -1 and (
                    index_find + len(searched) == len(description) or
                    description[index_find + len(searched)] == " "):
                if description.startswith("Minifig "):
                    description = description[8:]
                    if description.startswith("(") and description.endswith(")"):
                        description = description[1:-1]
                    section[description] = code
                break
        else:
            # The accessories are those Minifig items which do not fall into any
            # of the above categories.
            if description.startswith("Minifig "):
                description = description[8:]
                if description.startswith("(") and description.endswith(")"):
                    description = description[1:-1]
                self.parts['minifig']['accessories'][description] = code
            else:
                self.parts['others'][description] = code
        return code, description

    def part(self, description=None, code=None):
        """
        Gets a Part from its description or code
        :param description:
        :param code:
        :return:
        """
        if not self.path:
            return None
        if description:
            try:
                code = self.parts_by_name[description]
            except KeyError:
                return None
        elif not code:
            return None
        return self._load_part(code)

    def _find_parts_subdirs(self, directory):
        for item in os.listdir(directory):
            obj = os.path.join(directory, item)
            if os.path.isdir(obj):
                self.parts_subdirs[item] = obj
                self.parts_subdirs[item.lower()] = obj
                self.parts_subdirs[item.upper()] = obj

    def _load_part(self, code):
        code = code.replace("\\", os.sep)
        code = code.replace("/", os.sep)
        if os.sep in code:
            pieces = code.split(os.sep)
            if len(pieces) != 2:
                return None
            try:
                parts_dirs = [self.parts_subdirs[pieces[0]]]
            except KeyError:
                return None
            code = pieces[1]
        else:
            parts_dirs = self.parts_dirs
        paths = []
        for parts_dir in parts_dirs:
            paths.append(os.path.join(parts_dir, code.lower()) + os.extsep + "dat")
            paths.append(os.path.join(parts_dir, code.upper()) + os.extsep + "DAT")
        for path in paths:
            try:
                part = Part(path)
            except PartError:
                continue
            else:
                return part
        return None

    def _load_colours(self, path):
        try:
            colours_part = Part(path)
        except PartError:
            return
        for obj in colours_part.objects:
            if not isinstance(obj, MetaCommand) or not obj.text.startswith("!COLOUR"):
                continue
            pieces = obj.text.split()
            try:
                name = pieces[1]
                code = int(pieces[pieces.index("CODE") + 1])
                rgb = pieces[pieces.index("VALUE") + 1]

                self.colours[name] = rgb
                self.colours[code] = rgb

                colour = Colour(code, name, rgb)
                self.colours_by_name[name] = colour
                self.colours_by_code[code] = colour

            except (ValueError, IndexError):
                continue
            try:
                alpha_at = pieces.index("ALPHA")
                alpha = int(pieces[alpha_at + 1])
                self.alpha_values[name] = alpha
                self.alpha_values[code] = alpha
            except (IndexError, ValueError):
                pass

            colour_attributes = []
            for attribute in Parts.ColourAttributes:
                if attribute in pieces:
                    colour_attributes.append(attribute)

            self.colour_attributes[name] = colour_attributes
            self.colour_attributes[code] = colour_attributes

            alpha = self.alpha_values.get(name, 255)
            colour = Colour(code, name, rgb,
                            alpha, colour_attributes)
            self.colours_by_name[name] = colour
            self.colours_by_code[code] = colour

    def _load_primitives(self, path):
        try:
            part_path = codecs.open(path, 'r', encoding='utf-8')
            for line in part_path.readlines():
                pieces = re.split(DOT_DAT, line)
                if len(pieces) != 2:
                    break
                code = pieces[0]
                description = pieces[1].strip()
                self.primitives_by_name[description] = code
                self.primitives_by_code[code] = description
        except IOError:
            raise PartError("Failed to load primitives file: %s" % path)


def colour_from_str(colour_str):
    """ gets a Colour from a string """
    try:
        return int(colour_str)
    except ValueError:
        if colour_str.startswith('0x2'):
            return Colour(rgb="#" + colour_str[3:], alpha=255)


def _comment_or_meta(pieces):
    if not pieces:
        return Comment("")
    elif pieces[0][:1] == "!":
        return MetaCommand(" ".join(pieces))
    return Comment(" ".join(pieces))


def _sub_file(pieces):
    if len(pieces) != 14:
        raise PartError("Invalid part data")
    colour = colour_from_str(pieces[0])
    position = list(map(float, pieces[1:4]))
    rows = [list(map(float, pieces[4:7])),
            list(map(float, pieces[7:10])),
            list(map(float, pieces[10:13]))]
    part = pieces[13].upper()
    if re.search(ENDS_DOT_DAT, part):
        part = part[:-4]
    return Piece(Colour(colour), Vector(*position), Matrix(rows), part)


def _line(pieces):
    if len(pieces) != 7:
        raise PartError("Invalid line data")
    colour = colour_from_str(pieces[0])
    point1 = map(float, pieces[1:4])
    point2 = map(float, pieces[4:7])
    return Line(Colour(colour), Vector(*point1), Vector(*point2))


def _triangle(pieces):
    if len(pieces) != 10:
        raise PartError("Invalid triangle data")
    colour = colour_from_str(pieces[0])
    point1 = map(float, pieces[1:4])
    point2 = map(float, pieces[4:7])
    point3 = map(float, pieces[7:10])
    return Triangle(Colour(colour), Vector(*point1), Vector(*point2), Vector(*point3))


def _quadrilateral(pieces):
    if len(pieces) != 13:
        raise PartError("Invalid quadrilateral data")
    colour = colour_from_str(pieces[0])
    point1 = map(float, pieces[1:4])
    point2 = map(float, pieces[4:7])
    point3 = map(float, pieces[7:10])
    point4 = map(float, pieces[10:13])
    return Quadrilateral(Colour(colour), Vector(*point1), Vector(*point2),
                         Vector(*point3), Vector(*point4))


def _optional_line(pieces):
    if len(pieces) != 13:
        raise PartError("Invalid line data")
    colour = colour_from_str(pieces[0])
    point1 = map(float, pieces[1:4])
    point2 = map(float, pieces[4:7])
    point3 = map(float, pieces[7:10])
    point4 = map(float, pieces[10:13])
    return OptionalLine(Colour(colour), Vector(*point1), Vector(*point2),
                        Vector(*point3), Vector(*point4))


class Part(object):
    """
    Contains data from a LDraw part file
    """
    def __init__(self, path):
        self._handlers = {
            "0": _comment_or_meta,
            "1": _sub_file,
            "2": _line,
            "3": _triangle,
            "4": _quadrilateral,
            "5": _optional_line
        }
        self.path = path
        self.objects = []

        """ Load the Part from its path """
        try:
            lines = codecs.open(path, 'r', encoding='utf-8').readlines()
        except IOError:
            raise PartError("Failed to read part file: %s" % path)
        number = 0
        for line in lines:
            number += 1
            pieces = line.split()
            if not pieces:
                # self.objects.append(BlankLine)
                continue
            try:
                handler = self._handlers[pieces[0]]
            except KeyError:
                raise PartError("Unknown command (%s) in %s at line %i" % (path, pieces[0], number))
            try:
                obj = handler(pieces[1:])
                self.objects.append(obj)
            except PartError as parse_error:
                raise PartError(parse_error.message + ' in %s at line %i' % (self.path, number))



# coding=utf-8
"""
library/parts.py - Auto-generated Part classes for the Python ldraw package.

Copyright (C) 2018 Matthieu Berthomé <rienafairefr@gmail.com>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


<< This file is auto-generated, you shouldn't have to modify it>>
"""
# pylint: disable=invalid-name, line-too-long, too-many-lines

FoodTurkeyLeg = "33057"
LegLeft = "3817b"
LegLeftWith2SafetyPinsPattern = "3817pc44"
LegLeftWith_Emmet_Badge_ReflectiveStripePattern = "3817p8h"
LegLeftWith_Emmet_Badge_ReflectiveStripeAndSilverTrianglesOnFeetPattern = "3817p8i"
LegLeftWith2RedDiamondsPattern = "3817pba"
LegLeftWithAstroPattern = "3817p6f"
LegLeftWithBlue_WhiteTriangles_FringePattern = "3817pw2"
LegLeftWithButtonedPocketPattern = "3817pa3"
LegLeftWithClockworkRobotPattern = "3817pc67"
LegLeftWithDarkRedLoinclothAndWhiteClawsPattern = "3817paw"
LegLeftWithDarkTurquoise_WhiteTriangles_WhiteFringePattern = "3817pw3"
LegLeftWithDarkblueLoinclothPattern = "3817pq0"
LegLeftWithDarkblueAndGoldLoinclothPattern = "3817pq1"
LegLeftWithGoldenCircuitPattern = "3817p6w"
LegLeftWithGrassSkirtPattern = "3817p3j"
LegLeftWithGreenKiltAndToesPattern = "3817pa2"
LegLeftWithHolsterAndBeltPattern = "3817pa9"
LegLeftWithIronManArmouredSuitMarkViPattern = "3817pbb"
LegLeftWithIronManArmouredSuitMarkViiPattern = "3817pbc"
LegLeftWithIronManArmouredSuitMarkXliiPattern = "3817pbf"
LegLeftWithIronManArmouredSuitMarkXviiPattern = "3817pbe"
LegLeftWithIronPatriotArmouredSuitKneepadPattern = "3817pbg"
LegLeftWithLotrCoatAndShirttailsPattern = "3817pm2"
LegLeftWithLotrLeatherArmorPattern = "3817pm0"
LegLeftWithLotrScaleArmorPattern = "3817pm1"
LegLeftWithLaboratorySmockPattern = "3817pde"
LegLeftWithLeatherStraps_RedStuds_Pattern = "3817p4f"
LegLeftWithOrangeCablePattern = "3817p6u"
LegLeftWithPurpleGreatcoatPattern = "3817phb"
LegLeftWithRedRomanTunicAndBeltPattern = "3817pc6a"
LegLeftWithRed_WhiteTrianglesPattern = "3817pw1"
LegLeftWithRobotPattern = "3817p63"
LegLeftWithSwStormtrooperPattern = "3817psk"
LegLeftWithSwWhiteLeggingsPattern = "3817ps0"
LegLeftWithSalmonCablePattern = "3817p6v"
LegLeftWithWarMachineArmouredSuitKneepadPattern = "3817pbd"
LegLeftWithWhiteApronPattern = "3817p89"
LegRight = "3816b"
LegRightWith3BlackDiamondsPattern = "3816pba"
LegRightWithAstroPattern = "3816p6f"
LegRightWithBlue_WhiteTriangles_FringePattern = "3816pw2"
LegRightWithButtonedPocketPattern = "3816pa3"
LegRightWithChainlinkAnd3SafetyPinsPattern = "3816pc44"
LegRightWithClockworkRobotPattern = "3816pc67"
LegRightWithDarkRedLoincloth_WhiteClawsAndFurTailPattern = "3816paw"
LegRightWithDarkTurquoise_WhiteTriangles_WhiteFringePattern = "3816pw3"
LegRightWithDarkblueLoinclothPattern = "3816pq0"
LegRightWithDarkblueAndGoldLoinclothPattern = "3816pq1"
LegRightWithGoldenCircuitPattern = "3816p6w"
LegRightWithGrassSkirtPattern = "3816p3j"
LegRightWithGreenKiltAndToesPattern = "3816pa2"
LegRightWithHolsterAndBeltPattern = "3816pa9"
LegRightWithIronManArmouredSuitMarkViPattern = "3816pbb"
LegRightWithIronManArmouredSuitMarkViiPattern = "3816pbc"
LegRightWithIronManArmouredSuitMarkXliiPattern = "3816pbf"
LegRightWithIronManArmouredSuitMarkXviiPattern = "3816pbe"
LegRightWithIronPatriotArmouredSuitKneepadPattern = "3816pbg"
LegRightWithLotrCoatAndShirttailsPattern = "3816pm2"
LegRightWithLotrLeatherArmorPattern = "3816pm0"
LegRightWithLotrScaleArmorPattern = "3816pm1"
LegRightWithLaboratorySmockPattern = "3816pde"
LegRightWithLeatherStraps_RedStuds_Pattern = "3816p4f"
LegRightWithOrangeCablePattern = "3816p6u"
LegRightWithPurpleGreatcoatPattern = "3816phb"
LegRightWithRedRomanTunicAndBeltPattern = "3816pc6a"
LegRightWithRed_WhiteTriangles_FringePattern = "3816pw1"
LegRightWithReflectiveStripePattern = "3816p8h"
LegRightWithReflectiveStripeAndTrianglesOnFeetPattern = "3816p8i"
LegRightWithRobotPattern = "3816p63"
LegRightWithSwGunbeltPattern = "3816ps5"
LegRightWithSwStormtrooperPattern = "3816psk"
LegRightWithSwWhiteLeggingsPattern = "3816ps0"
LegRightWithSalmonCablePattern = "3816p6v"
LegRightWithWarMachineArmouredSuitKneepadPattern = "3816pbd"
LegRightWithWhiteApronPattern = "3816p89"
LegRightWithWhiteAndGoldMarkingsPattern = "3816p6g"
LegWooden = "2532"
MechanicalLeg = "53984"
RobotLeg1X2X1_667WithPin = "17486"
RobotLeg1X2X2_333WithPin = "30362a"
SkeletonLeg = "6266"
SkeletonLegLong = "23769"
SkeletonLegWithBlackSquareFoot = "93062p01"
SkeletonLegWithPearlDarkGreySquareFoot = "93062p02"
SkeletonLegWithSquareFoot = "93062"



from ldraw.library.parts.others import Brick2X2
from ldraw.pieces import Group, Piece
from ldraw.figure import *
from ldraw.geometry import Identity
from ldraw.library.colours import *
import numpy as np

class Dimensions():
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z

class LegoDimensions():
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z

    def convertToLdrDimensions(self):
        return self.x * 20, self.y * -8, self.z * 20

class BaseBlock():
    def __init__(self, x=5, y=5, z=10):
        self.group = Group()
        self.ldrContent = ""
        self.maxLdrDimensions = Dimensions(200, 120, 280)
        self.maxLegoDimensions = Dimensions(200, 120, 280)

    def append(self, newLine):
        self.ldrContent += newLine + "\n"

class LegoWorld():
    def __init__(self, x=4, y=3, z=4):
        self.group = Group()
        self.ldrContent = ""
        self.maxLegoDimensions= LegoDimensions(x, y, z)
        self.content = np.zeros([x,y,z])
        self.noOfPieces = 0
        self.yMap = np.zeros([x,z])
        self.yGlobalMax = 0
    
    def reset(self):
        self.content = np.zeros([self.maxLegoDimensions.x,self.maxLegoDimensions.y,self.maxLegoDimensions.z])
        self.ldrContent = ""
        self.noOfPieces = 0
        self.yMap = np.zeros([self.maxLegoDimensions.x, self.maxLegoDimensions.z])
        self.yGlobalMax = 0
        
    def findMaximumYValue(self, part, x, y, z):
        yGlobalMax = 0
        yOffset = 0
        for i in range(part.sizeDimensions.x):
            for j in range(part.sizeDimensions.z):
                restart = True
                while restart == True:
                    yOffset = int(self.yMap[x + i,z + j])
                    yLocalMax = int(self.yMap[x + i,z + j])
                    restart = False
                    for k in range(part.sizeDimensions.y+1):
                        if self.content[ x + i, yOffset + k, z + j] == 1:
                            if yLocalMax <= yOffset+k:    
                                yLocalMax += 1
                                self.yMap[x + i, z + j] = yLocalMax
                                restart = True  
                    if yLocalMax > yGlobalMax:
                        yGlobalMax = yLocalMax
                    if yGlobalMax > self.yGlobalMax:
                        self.yGlobalMax = yGlobalMax
        return yGlobalMax         

    def addPartToWorld(self, part, x, z):
        x = int(x)
        z = int(z)
        y = 0
        y = self.findMaximumYValue(part, x, y, z)
        for i in range(part.sizeDimensions.x):
            for j in range(part.sizeDimensions.y):
                for k in range(part.sizeDimensions.z):
                    self.content[x+i,y+j,z+k] = 1           
        self.noOfPieces += 1
        partToAdd = part.create(x, y, z, self.group)
        self.appendToLdrFile(partToAdd)

    def appendToLdrFile(self, newLine):
        self.ldrContent += newLine + "\n"

class TwoXTwoBlock():
    def __init__(self):
        self.sizeDimensions = LegoDimensions(x=2, y=3, z=2)

    def create(self, x, y, z, group):
        
        positionDimensions = LegoDimensions(x=x, y=y, z=z)
        convertedX, convertedY, convertedZ = positionDimensions.convertToLdrDimensions()
        print("adding part: ", convertedX, convertedY, convertedZ)
        return Piece(Dark_Blue, Vector(x=convertedX, y=convertedY, z=convertedZ), Identity(), Brick2X2, group).__repr__()




"""
pieces.py - Classes representing pieces and groups for the ldraw Python package.

Copyright (C) 2008 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
# pylint: disable=too-many-arguments, too-few-public-methods
from ldraw.compat import reduce
from ldraw.geometry import Identity, Vector


class Piece(object):
    """
    a Piece, which is a Part with a certain colour
    at a certain position and rotation
    """
    def __init__(self, colour, position, matrix, part, group=None):
        self.position = position
        self.colour = colour
        self.matrix = matrix
        self.part = part.upper()
        self.group = group
        if group:
            group.add_piece(self)

    def __repr__(self):
        if self.group:
            position = self.group.position + self.group.matrix * self.position
            matrix = self.group.matrix * self.matrix
        else:
            position = self.position
            matrix = self.matrix
        tup = tuple(reduce(lambda row1, row2: row1 + row2, matrix.rows))
        return ("1 %i " % self.colour.code) + \
               ("%f " * 3) % (position.x, position.y, position.z) + \
               ("%f " * 9) % tup + \
               ("%s.DAT" % self.part)


class Group(object):
    """ a Group of Pieces """
    def __init__(self, position=Vector(0, 0, 0), matrix=Identity()):
        self.position = position
        self.matrix = matrix
        self.pieces = []

    def __repr__(self):
        text = []
        for piece in self.pieces:
            text.append(repr(piece))
        return "\n".join(text)

    def add_piece(self, piece):
        """ add a piece to the group """
        self.pieces.append(piece)
        if piece.group and piece.group != self:
            piece.group.remove_piece(piece)
        piece.group = self

    def remove_piece(self, piece):
        """ remove a piece from the group """
        self.pieces.remove(piece)
        piece.group = None



""" Some tools to convert ldr to other formats"""
import argparse

import sys

from ldraw.config import get_config
from ldraw.parts import Part, Parts, PartError
from ldraw.geometry import Vector, CoordinateSystem


def widthxheight(input_str):
    """ a type for a widthxheight image size """
    image_dimensions = input_str.split("x")
    if len(image_dimensions) != 2:
        raise argparse.ArgumentTypeError("Expecting widthxheight")
    return list(map(int, image_dimensions))


def vector_position(input_str):
    """ a type for comma separated vector """
    position = input_str.split(",")
    if len(position) != 3:
        raise argparse.ArgumentTypeError("Expecting comma-separated elements for the position")
    return Vector(*map(float, position))


def get_model(ldraw_path):
    """" get model from ldraw path """
    config = get_config()
    parts = Parts(config['parts.lst'])
    try:
        model = Part(ldraw_path)
    except PartError:
        sys.stderr.write("Failed to read LDraw file: %s\n" % ldraw_path)
        sys.exit(1)
    return model, parts


UP_DIRECTION = Vector(0, -1.0, 0)


def get_coordinate_system(camera_position, look_at_position):
    """" get coordinate system of the view """
    system = CoordinateSystem()
    system.z = (camera_position - look_at_position)
    system.z.norm()
    system.x = UP_DIRECTION.cross(system.z)
    if abs(system.x) == 0.0:
        system.x = system.z.cross(Vector(1.0, 0, 0))
    system.x.norm()
    system.y = system.z.cross(system.x)
    return system


def verify_camera_look_at(camera_position, look_at_position):
    """ verify that the camera and look_at positions are valid """
    if camera_position == look_at_position:
        sys.stderr.write("Camera and look-at positions are the same.\n")
        sys.exit(1)



"""
geometry.py - Geometry classes for the ldraw Python package.

Copyright (C) 2008 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
# pylint: disable=invalid-name, too-few-public-methods, missing-docstring
import copy
import math
from numbers import Number
from ldraw.compat import reduce


class MatrixError(Exception):
    pass


class Axis(object):
    pass


class XAxis(Axis):
    pass


class YAxis(Axis):
    pass


class ZAxis(Axis):
    pass


class AngleUnits(object):
    pass


class Radians(AngleUnits):
    pass


class Degrees(AngleUnits):
    pass


def _rows_multiplication(r1, r2):
    rows = [[r1[0][0] * r2[0][0] + r1[0][1] * r2[1][0] + r1[0][2] * r2[2][0],
             r1[0][0] * r2[0][1] + r1[0][1] * r2[1][1] + r1[0][2] * r2[2][1],
             r1[0][0] * r2[0][2] + r1[0][1] * r2[1][2] + r1[0][2] * r2[2][2]],
            [r1[1][0] * r2[0][0] + r1[1][1] * r2[1][0] + r1[1][2] * r2[2][0],
             r1[1][0] * r2[0][1] + r1[1][1] * r2[1][1] + r1[1][2] * r2[2][1],
             r1[1][0] * r2[0][2] + r1[1][1] * r2[1][2] + r1[1][2] * r2[2][2]],
            [r1[2][0] * r2[0][0] + r1[2][1] * r2[1][0] + r1[2][2] * r2[2][0],
             r1[2][0] * r2[0][1] + r1[2][1] * r2[1][1] + r1[2][2] * r2[2][1],
             r1[2][0] * r2[0][2] + r1[2][1] * r2[1][2] + r1[2][2] * r2[2][2]]]
    return rows


class Matrix(object):
    """ a transformation matrix """

    def __init__(self, rows):
        self.rows = rows

    def __repr__(self):
        values = reduce(lambda x, y: x + y, self.rows)
        format_string = ("((%f, %f, %f),\n"
                         " (%f, %f, %f),\n"
                         " (%f, %f, %f))")
        return format_string % tuple(values)

    def __mul__(self, other):
        if isinstance(other, Matrix):
            r1 = self.rows
            r2 = other.rows
            return Matrix(_rows_multiplication(r1, r2))
        elif isinstance(other, Vector):
            r = self.rows
            x, y, z = other.x, other.y, other.z
            return Vector(r[0][0] * x + r[0][1] * y + r[0][2] * z,
                          r[1][0] * x + r[1][1] * y + r[1][2] * z,
                          r[2][0] * x + r[2][1] * y + r[2][2] * z)
        else:
            raise MatrixError

    def __rmul__(self, other):
        if isinstance(other, Matrix):
            r1 = other.rows
            r2 = self.rows
            return Matrix(_rows_multiplication(r1, r2))
        elif isinstance(other, Vector):
            r = self.rows
            x, y, z = other.x, other.y, other.z
            return Vector(x * r[0][0] + y * r[1][0] + z * r[2][0],
                          x * r[0][1] + y * r[1][1] + z * r[2][1],
                          x * r[0][2] + y * r[1][2] + z * r[2][2])
        else:
            raise MatrixError

    def copy(self):
        """ make a copy of this matrix """
        return Matrix(copy.deepcopy(self.rows))

    def rotate(self, angle, axis, units=Degrees):
        """ rotate the matrix by an angle around an axis """
        if units == Degrees:
            c = math.cos(angle / 180.0 * math.pi)
            s = math.sin(angle / 180.0 * math.pi)
        else:
            c = math.cos(angle)
            s = math.sin(angle)
        if axis == XAxis:
            rotation = Matrix([[1, 0, 0], [0, c, -s], [0, s, c]])
        elif axis == YAxis:
            rotation = Matrix([[c, 0, -s], [0, 1, 0], [s, 0, c]])
        elif axis == ZAxis:
            rotation = Matrix([[c, -s, 0], [s, c, 0], [0, 0, 1]])
        else:
            raise MatrixError("Invalid axis specified.")
        return self * rotation

    def scale(self, sx, sy, sz):
        """ scale the matrix by a number"""
        return Matrix([[sx, 0, 0], [0, sy, 0], [0, 0, sz]]) * self

    def transpose(self):
        """ transpose """
        r = self.rows
        return Matrix([[r[0][0], r[1][0], r[2][0]],
                       [r[0][1], r[1][1], r[2][1]],
                       [r[0][2], r[1][2], r[2][2]]])

    def det(self):
        """ determinant of the matrix """
        r = self.rows
        terms = [r[0][0] * (r[1][1] * r[2][2] - r[1][2] * r[2][1]),
                 r[0][1] * (r[1][2] * r[2][0] - r[1][0] * r[2][2]),
                 r[0][2] * (r[1][0] * r[2][1] - r[1][1] * r[2][0])]
        return sum(terms)

    def flatten(self):
        """ flatten the matrix """
        return tuple(reduce(lambda x, y: x + y, self.rows))

    def fix_diagonal(self):
        """ POV-Ray does not like matrices with zero diagonal elements. """
        corrected = False
        for i in range(3):
            if self.rows[i][i] == 0.0:
                self.rows[i][i] = 0.001
                corrected = True
        return corrected

    def __eq__(self, other):
        if not isinstance(other, Matrix):
            return False
        return self.rows == other.rows


def Identity():
    """ a transformation matrix representing Identity """
    return Matrix([[1, 0, 0], [0, 1, 0], [0, 0, 1]])


class Vector(object):
    """ a Vector in 3D"""

    def __init__(self, x, y, z):
        self.x, self.y, self.z = x, y, z

    @property
    def repr(self):
        return "%f, %f, %f" % (self.x, self.y, self.z)

    def __repr__(self):
        return "<Vector: (%s)>" % (self.repr)

    def __hash__(self):
        return hash((self.x, self.y, self.z))

    def __add__(self, other):
        x = self.x + other.x
        y = self.y + other.y
        z = self.z + other.z
        # Return a new object.
        return Vector(x, y, z)

    __radd__ = __add__

    def __sub__(self, other):
        x = self.x - other.x
        y = self.y - other.y
        z = self.z - other.z
        # Return a new object.
        return Vector(x, y, z)

    def __rsub__(self, other):
        x = other.x - self.x
        y = other.y - self.y
        z = other.z - self.z
        # Return a new object.
        return Vector(x, y, z)

    def __cmp__(self, other):
        # This next expression will only return zero (equals) if all
        # expressions are false.
        return self.x != other.x or self.y != other.y or self.z != other.z

    def __eq__(self, other):
        if not isinstance(other, Vector):
            return False
        return self.x == other.x and self.y == other.y and self.z == other.z

    def __abs__(self):
        return (self.x ** 2 + self.y ** 2 + self.z ** 2) ** 0.5

    def __rmul__(self, other):
        if isinstance(other, Number):
            return Vector(self.x * other, self.y * other, self.z * other)
        raise ValueError("Cannot multiply %s with %s" % (self.__class__, type(other)))

    def __div__(self, other):
        if isinstance(other, Number):
            return Vector(self.x / other, self.y / other, self.z / other)
        raise ValueError("Cannot divide %s with %s" % (self.__class__, type(other)))

    def copy(self):
        """vector = copy(self)
        Copy the vector so that new vectors containing the same values
        are passed around rather than references to the same object.
        """
        return Vector(self.x, self.y, self.z)

    def cross(self, other):
        """ cross product """
        return Vector(self.y * other.z - self.z * other.y,
                      self.z * other.x - self.x * other.z,
                      self.x * other.y - self.y * other.x)

    def dot(self, other):
        """ dot product"""
        return self.x * other.x + self.y * other.y + self.z * other.z

    def norm(self):
        """ normalized """
        _length = abs(self)
        self.x = self.x / _length
        self.y = self.y / _length
        self.z = self.z / _length


class Vector2D(object):
    """ a Vector in 2D """

    def __init__(self, x, y):
        self.x, self.y = x, y

    def __repr__(self):
        return "<Vector2D: (%f, %f) >" % (self.x, self.y)

    def __hash__(self):
        return hash((self.x, self.y))

    def __add__(self, other):
        x = self.x + other.x
        y = self.y + other.y
        # Return a new object.
        return Vector2D(x, y)

    __radd__ = __add__

    def __sub__(self, other):
        x = self.x - other.x
        y = self.y - other.y
        # Return a new object.
        return Vector2D(x, y)

    def __rsub__(self, other):
        x = other.x - self.x
        y = other.y - self.y
        # Return a new object.
        return Vector2D(x, y)

    def __cmp__(self, other):
        # This next expression will only return zero (equals) if all
        # expressions are false.
        return self.x != other.x or self.y != other.y

    def __abs__(self):
        return (self.x ** 2 + self.y ** 2) ** 0.5

    def __rmul__(self, other):
        if isinstance(other, Number):
            return Vector2D(self.x * other, self.y * other)
        raise ValueError("Cannot multiply %s with %s" % (self.__class__, type(other)))

    def __div__(self, other):
        if isinstance(other, Number):
            return Vector2D(self.x / other, self.y / other)
        raise ValueError("Cannot divide %s with %s" % (self.__class__, type(other)))

    def copy(self):
        """
        vector = copy(self)
        Copy the vector so that new vectors containing the same values
        are passed around rather than references to the same object.
        """
        return Vector2D(self.x, self.y)

    def dot(self, other):
        """ dot product """
        return self.x * other.x + self.y * other.y


class CoordinateSystem(object):
    def __init__(self, x=Vector(1.0, 0.0, 0.0),
                 y=Vector(0.0, 1.0, 0.0),
                 z=Vector(0.0, 0.0, 1.0)):
        self.x = x
        self.y = y
        self.z = z

    def project(self, p):
        return Vector(p.dot(self.x), p.dot(self.y), p.dot(self.z))



# coding=utf-8
"""
colours.py - Colour definitions for the Python ldraw package.

Copyright (C) 2008 David Boddie <david@boddie.org.uk>
(C) 2018 Matthieu Berthomé <rienafairefr@gmail.com>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
<< Autogenerated source file >>
"""
# pylint: disable=invalid-name
from ldraw.colour import Colour

Black = Colour(0, "Black", "#05131D", 255, [])
Blue = Colour(1, "Blue", "#0055BF", 255, [])
Green = Colour(2, "Green", "#257A3E", 255, [])
Dark_Turquoise = Colour(3, "Dark_Turquoise", "#00838F", 255, [])
Red = Colour(4, "Red", "#C91A09", 255, [])
Dark_Pink = Colour(5, "Dark_Pink", "#C870A0", 255, [])
Brown = Colour(6, "Brown", "#583927", 255, [])
Light_Grey = Colour(7, "Light_Grey", "#9BA19D", 255, [])
Dark_Grey = Colour(8, "Dark_Grey", "#6D6E5C", 255, [])
Light_Blue = Colour(9, "Light_Blue", "#B4D2E3", 255, [])
Bright_Green = Colour(10, "Bright_Green", "#4B9F4A", 255, [])
Light_Turquoise = Colour(11, "Light_Turquoise", "#55A5AF", 255, [])
Salmon = Colour(12, "Salmon", "#F2705E", 255, [])
Pink = Colour(13, "Pink", "#FC97AC", 255, [])
Yellow = Colour(14, "Yellow", "#F2CD37", 255, [])
White = Colour(15, "White", "#FFFFFF", 255, [])
Main_Colour = Colour(16, "Main_Colour", "#FFFF80", 255, [])
Light_Green = Colour(17, "Light_Green", "#C2DAB8", 255, [])
Light_Yellow = Colour(18, "Light_Yellow", "#FBE696", 255, [])
Tan = Colour(19, "Tan", "#E4CD9E", 255, [])
Light_Violet = Colour(20, "Light_Violet", "#C9CAE2", 255, [])
Glow_In_Dark_Opaque = Colour(21, "Glow_In_Dark_Opaque", "#E0FFB0", 240, [])
Purple = Colour(22, "Purple", "#81007B", 255, [])
Dark_Blue_Violet = Colour(23, "Dark_Blue_Violet", "#2032B0", 255, [])
Edge_Colour = Colour(24, "Edge_Colour", "#7F7F7F", 255, [])
Orange = Colour(25, "Orange", "#FE8A18", 255, [])
Magenta = Colour(26, "Magenta", "#923978", 255, [])
Lime = Colour(27, "Lime", "#BBE90B", 255, [])
Dark_Tan = Colour(28, "Dark_Tan", "#958A73", 255, [])
Bright_Pink = Colour(29, "Bright_Pink", "#E4ADC8", 255, [])
Medium_Lavender = Colour(30, "Medium_Lavender", "#AC78BA", 255, [])
Lavender = Colour(31, "Lavender", "#E1D5ED", 255, [])
Trans_Black_Ir_Lens = Colour(32, "Trans_Black_Ir_Lens", "#000000", 210, [])
Trans_Dark_Blue = Colour(33, "Trans_Dark_Blue", "#0020A0", 128, [])
Trans_Green = Colour(34, "Trans_Green", "#237841", 128, [])
Trans_Bright_Green = Colour(35, "Trans_Bright_Green", "#56E646", 128, [])
Trans_Red = Colour(36, "Trans_Red", "#C91A09", 128, [])
Trans_Dark_Pink = Colour(37, "Trans_Dark_Pink", "#DF6695", 128, [])
Trans_Neon_Orange = Colour(38, "Trans_Neon_Orange", "#FF800D", 128, [])
Trans_Very_Light_Blue = Colour(39, "Trans_Very_Light_Blue", "#C1DFF0", 128, [])
Trans_Black = Colour(40, "Trans_Black", "#635F52", 128, [])
Trans_Medium_Blue = Colour(41, "Trans_Medium_Blue", "#559AB7", 128, [])
Trans_Neon_Green = Colour(42, "Trans_Neon_Green", "#C0FF00", 128, [])
Trans_Light_Blue = Colour(43, "Trans_Light_Blue", "#AEE9EF", 128, [])
Trans_Bright_Reddish_Lilac = Colour(44, "Trans_Bright_Reddish_Lilac", "#96709F", 128, [])
Trans_Pink = Colour(45, "Trans_Pink", "#FC97AC", 128, [])
Trans_Yellow = Colour(46, "Trans_Yellow", "#F5CD2F", 128, [])
Trans_Clear = Colour(47, "Trans_Clear", "#FCFCFC", 128, [])
Trans_Purple = Colour(52, "Trans_Purple", "#A5A5CB", 128, [])
Trans_Neon_Yellow = Colour(54, "Trans_Neon_Yellow", "#DAB000", 128, [])
Trans_Orange = Colour(57, "Trans_Orange", "#F08F1C", 128, [])
Chrome_Antique_Brass = Colour(60, "Chrome_Antique_Brass", "#645A4C", 255, ['CHROME'])
Chrome_Blue = Colour(61, "Chrome_Blue", "#6C96BF", 255, ['CHROME'])
Chrome_Green = Colour(62, "Chrome_Green", "#3CB371", 255, ['CHROME'])
Chrome_Pink = Colour(63, "Chrome_Pink", "#AA4D8E", 255, ['CHROME'])
Chrome_Black = Colour(64, "Chrome_Black", "#1B2A34", 255, ['CHROME'])
Rubber_Yellow = Colour(65, "Rubber_Yellow", "#F5CD2F", 255, ['RUBBER'])
Rubber_Trans_Yellow = Colour(66, "Rubber_Trans_Yellow", "#CAB000", 128, ['RUBBER'])
Rubber_Trans_Clear = Colour(67, "Rubber_Trans_Clear", "#FFFFFF", 128, ['RUBBER'])
Very_Light_Orange = Colour(68, "Very_Light_Orange", "#F3CF9B", 255, [])
Bright_Reddish_Lilac = Colour(69, "Bright_Reddish_Lilac", "#CD6298", 255, [])
Reddish_Brown = Colour(70, "Reddish_Brown", "#582A12", 255, [])
Light_Bluish_Grey = Colour(71, "Light_Bluish_Grey", "#A0A5A9", 255, [])
Dark_Bluish_Grey = Colour(72, "Dark_Bluish_Grey", "#6C6E68", 255, [])
Medium_Blue = Colour(73, "Medium_Blue", "#5C9DD1", 255, [])
Medium_Green = Colour(74, "Medium_Green", "#73DCA1", 255, [])
Speckle_Black_Copper = Colour(75, "Speckle_Black_Copper", "#000000", 255, [])
Speckle_Dark_Bluish_Grey_Silver = Colour(76, "Speckle_Dark_Bluish_Grey_Silver", "#635F61", 255, [])
Light_Pink = Colour(77, "Light_Pink", "#FECCCF", 255, [])
Light_Flesh = Colour(78, "Light_Flesh", "#F6D7B3", 255, [])
Milky_White = Colour(79, "Milky_White", "#FFFFFF", 240, [])
Metallic_Silver = Colour(80, "Metallic_Silver", "#A5A9B4", 255, ['METAL'])
Metallic_Green = Colour(81, "Metallic_Green", "#899B5F", 255, ['METAL'])
Metallic_Gold = Colour(82, "Metallic_Gold", "#DBAC34", 255, ['METAL'])
Metallic_Black = Colour(83, "Metallic_Black", "#1A2831", 255, ['METAL'])
Medium_Dark_Flesh = Colour(84, "Medium_Dark_Flesh", "#CC702A", 255, [])
Medium_Lilac = Colour(85, "Medium_Lilac", "#3F3691", 255, [])
Dark_Flesh = Colour(86, "Dark_Flesh", "#7C503A", 255, [])
Metallic_Dark_Grey = Colour(87, "Metallic_Dark_Grey", "#6D6E5C", 255, ['METAL'])
Blue_Violet = Colour(89, "Blue_Violet", "#4C61DB", 255, [])
Flesh = Colour(92, "Flesh", "#D09168", 255, [])
Light_Salmon = Colour(100, "Light_Salmon", "#FEBABD", 255, [])
Violet = Colour(110, "Violet", "#4354A3", 255, [])
Medium_Violet = Colour(112, "Medium_Violet", "#6874CA", 255, [])
Glitter_Trans_Dark_Pink = Colour(114, "Glitter_Trans_Dark_Pink", "#DF6695", 128, [])
Medium_Lime = Colour(115, "Medium_Lime", "#C7D23C", 255, [])
Glitter_Trans_Clear = Colour(117, "Glitter_Trans_Clear", "#FFFFFF", 128, [])
Aqua = Colour(118, "Aqua", "#B3D7D1", 255, [])
Light_Lime = Colour(120, "Light_Lime", "#D9E4A7", 255, [])
Light_Orange = Colour(125, "Light_Orange", "#F9BA61", 255, [])
Dark_Nougat = Colour(128, "Dark_Nougat", "#AD6140", 255, [])
Glitter_Trans_Purple = Colour(129, "Glitter_Trans_Purple", "#640061", 128, [])
Speckle_Black_Silver = Colour(132, "Speckle_Black_Silver", "#000000", 255, [])
Speckle_Black_Gold = Colour(133, "Speckle_Black_Gold", "#000000", 255, [])
Copper = Colour(134, "Copper", "#964A27", 255, ['PEARLESCENT'])
Pearl_Light_Grey = Colour(135, "Pearl_Light_Grey", "#9CA3A8", 255, ['PEARLESCENT'])
Metal_Blue = Colour(137, "Metal_Blue", "#5677BA", 255, ['PEARLESCENT'])
Pearl_Light_Gold = Colour(142, "Pearl_Light_Gold", "#DCBE61", 255, ['PEARLESCENT'])
Pearl_Dark_Grey = Colour(148, "Pearl_Dark_Grey", "#575857", 255, ['PEARLESCENT'])
Pearl_Very_Light_Grey = Colour(150, "Pearl_Very_Light_Grey", "#BBBDBC", 255, ['PEARLESCENT'])
Very_Light_Bluish_Grey = Colour(151, "Very_Light_Bluish_Grey", "#E6E3E0", 255, [])
Flat_Dark_Gold = Colour(178, "Flat_Dark_Gold", "#B4883E", 255, ['PEARLESCENT'])
Flat_Silver = Colour(179, "Flat_Silver", "#898788", 255, ['PEARLESCENT'])
Pearl_White = Colour(183, "Pearl_White", "#F2F3F2", 255, ['PEARLESCENT'])
Metallic_Bright_Red = Colour(184, "Metallic_Bright_Red", "#D60026", 255, ['METAL'])
Metallic_Dark_Green = Colour(186, "Metallic_Dark_Green", "#008E3C", 255, ['METAL'])
Reddish_Gold = Colour(189, "Reddish_Gold", "#AC8247", 255, ['PEARLESCENT'])
Bright_Light_Orange = Colour(191, "Bright_Light_Orange", "#F8BB3D", 255, [])
Bright_Light_Blue = Colour(212, "Bright_Light_Blue", "#86C1E1", 255, [])
Rust = Colour(216, "Rust", "#B31004", 255, [])
Reddish_Lilac = Colour(218, "Reddish_Lilac", "#8E5597", 255, [])
Lilac = Colour(219, "Lilac", "#564E9D", 255, [])
Bright_Light_Yellow = Colour(226, "Bright_Light_Yellow", "#FFF03A", 255, [])
Trans_Bright_Light_Orange = Colour(231, "Trans_Bright_Light_Orange", "#FCB76D", 128, [])
Sky_Blue = Colour(232, "Sky_Blue", "#56BED6", 255, [])
Trans_Fire_Yellow = Colour(234, "Trans_Fire_Yellow", "#FBE890", 128, [])
Rubber_Black = Colour(256, "Rubber_Black", "#212121", 255, ['RUBBER'])
Dark_Blue = Colour(272, "Dark_Blue", "#0D325B", 255, [])
Rubber_Blue = Colour(273, "Rubber_Blue", "#0033B2", 255, ['RUBBER'])
Trans_Reddish_Lilac = Colour(284, "Trans_Reddish_Lilac", "#C281A5", 128, [])
Trans_Light_Green = Colour(285, "Trans_Light_Green", "#7DC291", 128, [])
Dark_Green = Colour(288, "Dark_Green", "#184632", 255, [])
Trans_Light_Blue_Violet = Colour(293, "Trans_Light_Blue_Violet", "#6BABE4", 128, [])
Glow_In_Dark_Trans = Colour(294, "Glow_In_Dark_Trans", "#BDC6AD", 240, [])
Flamingo_Pink = Colour(295, "Flamingo_Pink", "#FF94C2", 255, [])
Pearl_Gold = Colour(297, "Pearl_Gold", "#CC9C2B", 255, ['PEARLESCENT'])
Metallic_Copper = Colour(300, "Metallic_Copper", "#C27F53", 255, ['METAL'])
Glitter_Trans_Light_Blue = Colour(302, "Glitter_Trans_Light_Blue", "#AEE9EF", 128, [])
Dark_Brown = Colour(308, "Dark_Brown", "#352100", 255, [])
Maersk_Blue = Colour(313, "Maersk_Blue", "#54A9C8", 255, [])
Dark_Red = Colour(320, "Dark_Red", "#720E0F", 255, [])
Dark_Azure = Colour(321, "Dark_Azure", "#1498D7", 255, [])
Medium_Azure = Colour(322, "Medium_Azure", "#3EC2DD", 255, [])
Light_Aqua = Colour(323, "Light_Aqua", "#BDDCD8", 255, [])
Rubber_Red = Colour(324, "Rubber_Red", "#C40026", 255, ['RUBBER'])
Yellowish_Green = Colour(326, "Yellowish_Green", "#DFEEA5", 255, [])
Glow_In_Dark_White = Colour(329, "Glow_In_Dark_White", "#F5F3D7", 240, [])
Olive_Green = Colour(330, "Olive_Green", "#9B9A5A", 255, [])
Chrome_Gold = Colour(334, "Chrome_Gold", "#BBA53D", 255, ['CHROME'])
Sand_Red = Colour(335, "Sand_Red", "#D67572", 255, [])
Glitter_Trans_Neon_Green = Colour(339, "Glitter_Trans_Neon_Green", "#C0FF00", 128, [])
Rubber_Orange = Colour(350, "Rubber_Orange", "#D06610", 255, ['RUBBER'])
Medium_Dark_Pink = Colour(351, "Medium_Dark_Pink", "#F785B1", 255, [])
Earth_Orange = Colour(366, "Earth_Orange", "#FA9C1C", 255, [])
Sand_Purple = Colour(373, "Sand_Purple", "#845E84", 255, [])
Rubber_Light_Grey = Colour(375, "Rubber_Light_Grey", "#C1C2C1", 255, ['RUBBER'])
Sand_Green = Colour(378, "Sand_Green", "#A0BCAC", 255, [])
Sand_Blue = Colour(379, "Sand_Blue", "#597184", 255, [])
Chrome_Silver = Colour(383, "Chrome_Silver", "#E0E0E0", 255, ['CHROME'])
Rubber_Dark_Blue = Colour(406, "Rubber_Dark_Blue", "#001D68", 255, ['RUBBER'])
Rubber_Purple = Colour(449, "Rubber_Purple", "#81007B", 255, ['RUBBER'])
Fabuland_Brown = Colour(450, "Fabuland_Brown", "#B67B50", 255, [])
Medium_Orange = Colour(462, "Medium_Orange", "#FFA70B", 255, [])
Dark_Orange = Colour(484, "Dark_Orange", "#A95500", 255, [])
Rubber_Lime = Colour(490, "Rubber_Lime", "#D7F000", 255, ['RUBBER'])
Magnet = Colour(493, "Magnet", "#656761", 255, ['METAL'])
Electric_Contact_Alloy = Colour(494, "Electric_Contact_Alloy", "#D0D0D0", 255, ['METAL'])
Electric_Contact_Copper = Colour(495, "Electric_Contact_Copper", "#AE7A59", 255, ['METAL'])
Rubber_Light_Bluish_Grey = Colour(496, "Rubber_Light_Bluish_Grey", "#A3A2A4", 255, ['RUBBER'])
Very_Light_Grey = Colour(503, "Very_Light_Grey", "#E6E3DA", 255, [])
Rubber_Flat_Silver = Colour(504, "Rubber_Flat_Silver", "#898788", 255, ['RUBBER'])
Rubber_White = Colour(511, "Rubber_White", "#FAFAFA", 255, ['RUBBER'])

ColoursByCode = {
    0: Black,
    1: Blue,
    2: Green,
    3: Dark_Turquoise,
    4: Red,
    5: Dark_Pink,
    6: Brown,
    7: Light_Grey,
    8: Dark_Grey,
    9: Light_Blue,
    10: Bright_Green,
    11: Light_Turquoise,
    12: Salmon,
    13: Pink,
    14: Yellow,
    15: White,
    16: Main_Colour,
    17: Light_Green,
    18: Light_Yellow,
    19: Tan,
    20: Light_Violet,
    21: Glow_In_Dark_Opaque,
    22: Purple,
    23: Dark_Blue_Violet,
    24: Edge_Colour,
    25: Orange,
    26: Magenta,
    27: Lime,
    28: Dark_Tan,
    29: Bright_Pink,
    30: Medium_Lavender,
    31: Lavender,
    32: Trans_Black_Ir_Lens,
    33: Trans_Dark_Blue,
    34: Trans_Green,
    35: Trans_Bright_Green,
    36: Trans_Red,
    37: Trans_Dark_Pink,
    38: Trans_Neon_Orange,
    39: Trans_Very_Light_Blue,
    40: Trans_Black,
    41: Trans_Medium_Blue,
    42: Trans_Neon_Green,
    43: Trans_Light_Blue,
    44: Trans_Bright_Reddish_Lilac,
    45: Trans_Pink,
    46: Trans_Yellow,
    47: Trans_Clear,
    52: Trans_Purple,
    54: Trans_Neon_Yellow,
    57: Trans_Orange,
    60: Chrome_Antique_Brass,
    61: Chrome_Blue,
    62: Chrome_Green,
    63: Chrome_Pink,
    64: Chrome_Black,
    65: Rubber_Yellow,
    66: Rubber_Trans_Yellow,
    67: Rubber_Trans_Clear,
    68: Very_Light_Orange,
    69: Bright_Reddish_Lilac,
    70: Reddish_Brown,
    71: Light_Bluish_Grey,
    72: Dark_Bluish_Grey,
    73: Medium_Blue,
    74: Medium_Green,
    75: Speckle_Black_Copper,
    76: Speckle_Dark_Bluish_Grey_Silver,
    77: Light_Pink,
    78: Light_Flesh,
    79: Milky_White,
    80: Metallic_Silver,
    81: Metallic_Green,
    82: Metallic_Gold,
    83: Metallic_Black,
    84: Medium_Dark_Flesh,
    85: Medium_Lilac,
    86: Dark_Flesh,
    87: Metallic_Dark_Grey,
    89: Blue_Violet,
    92: Flesh,
    100: Light_Salmon,
    110: Violet,
    112: Medium_Violet,
    114: Glitter_Trans_Dark_Pink,
    115: Medium_Lime,
    117: Glitter_Trans_Clear,
    118: Aqua,
    120: Light_Lime,
    125: Light_Orange,
    128: Dark_Nougat,
    129: Glitter_Trans_Purple,
    132: Speckle_Black_Silver,
    133: Speckle_Black_Gold,
    134: Copper,
    135: Pearl_Light_Grey,
    137: Metal_Blue,
    142: Pearl_Light_Gold,
    148: Pearl_Dark_Grey,
    150: Pearl_Very_Light_Grey,
    151: Very_Light_Bluish_Grey,
    178: Flat_Dark_Gold,
    179: Flat_Silver,
    183: Pearl_White,
    184: Metallic_Bright_Red,
    186: Metallic_Dark_Green,
    189: Reddish_Gold,
    191: Bright_Light_Orange,
    212: Bright_Light_Blue,
    216: Rust,
    218: Reddish_Lilac,
    219: Lilac,
    226: Bright_Light_Yellow,
    231: Trans_Bright_Light_Orange,
    232: Sky_Blue,
    234: Trans_Fire_Yellow,
    256: Rubber_Black,
    272: Dark_Blue,
    273: Rubber_Blue,
    284: Trans_Reddish_Lilac,
    285: Trans_Light_Green,
    288: Dark_Green,
    293: Trans_Light_Blue_Violet,
    294: Glow_In_Dark_Trans,
    295: Flamingo_Pink,
    297: Pearl_Gold,
    300: Metallic_Copper,
    302: Glitter_Trans_Light_Blue,
    308: Dark_Brown,
    313: Maersk_Blue,
    320: Dark_Red,
    321: Dark_Azure,
    322: Medium_Azure,
    323: Light_Aqua,
    324: Rubber_Red,
    326: Yellowish_Green,
    329: Glow_In_Dark_White,
    330: Olive_Green,
    334: Chrome_Gold,
    335: Sand_Red,
    339: Glitter_Trans_Neon_Green,
    350: Rubber_Orange,
    351: Medium_Dark_Pink,
    366: Earth_Orange,
    373: Sand_Purple,
    375: Rubber_Light_Grey,
    378: Sand_Green,
    379: Sand_Blue,
    383: Chrome_Silver,
    406: Rubber_Dark_Blue,
    449: Rubber_Purple,
    450: Fabuland_Brown,
    462: Medium_Orange,
    484: Dark_Orange,
    490: Rubber_Lime,
    493: Magnet,
    494: Electric_Contact_Alloy,
    495: Electric_Contact_Copper,
    496: Rubber_Light_Bluish_Grey,
    503: Very_Light_Grey,
    504: Rubber_Flat_Silver,
    511: Rubber_White,
}

ColoursByName = {
    "Black": Black,
    "Blue": Blue,
    "Green": Green,
    "Dark_Turquoise": Dark_Turquoise,
    "Red": Red,
    "Dark_Pink": Dark_Pink,
    "Brown": Brown,
    "Light_Grey": Light_Grey,
    "Dark_Grey": Dark_Grey,
    "Light_Blue": Light_Blue,
    "Bright_Green": Bright_Green,
    "Light_Turquoise": Light_Turquoise,
    "Salmon": Salmon,
    "Pink": Pink,
    "Yellow": Yellow,
    "White": White,
    "Main_Colour": Main_Colour,
    "Light_Green": Light_Green,
    "Light_Yellow": Light_Yellow,
    "Tan": Tan,
    "Light_Violet": Light_Violet,
    "Glow_In_Dark_Opaque": Glow_In_Dark_Opaque,
    "Purple": Purple,
    "Dark_Blue_Violet": Dark_Blue_Violet,
    "Edge_Colour": Edge_Colour,
    "Orange": Orange,
    "Magenta": Magenta,
    "Lime": Lime,
    "Dark_Tan": Dark_Tan,
    "Bright_Pink": Bright_Pink,
    "Medium_Lavender": Medium_Lavender,
    "Lavender": Lavender,
    "Trans_Black_IR_Lens": Trans_Black_Ir_Lens,
    "Trans_Dark_Blue": Trans_Dark_Blue,
    "Trans_Green": Trans_Green,
    "Trans_Bright_Green": Trans_Bright_Green,
    "Trans_Red": Trans_Red,
    "Trans_Dark_Pink": Trans_Dark_Pink,
    "Trans_Neon_Orange": Trans_Neon_Orange,
    "Trans_Very_Light_Blue": Trans_Very_Light_Blue,
    "Trans_Black": Trans_Black,
    "Trans_Medium_Blue": Trans_Medium_Blue,
    "Trans_Neon_Green": Trans_Neon_Green,
    "Trans_Light_Blue": Trans_Light_Blue,
    "Trans_Bright_Reddish_Lilac": Trans_Bright_Reddish_Lilac,
    "Trans_Pink": Trans_Pink,
    "Trans_Yellow": Trans_Yellow,
    "Trans_Clear": Trans_Clear,
    "Trans_Purple": Trans_Purple,
    "Trans_Neon_Yellow": Trans_Neon_Yellow,
    "Trans_Orange": Trans_Orange,
    "Chrome_Antique_Brass": Chrome_Antique_Brass,
    "Chrome_Blue": Chrome_Blue,
    "Chrome_Green": Chrome_Green,
    "Chrome_Pink": Chrome_Pink,
    "Chrome_Black": Chrome_Black,
    "Rubber_Yellow": Rubber_Yellow,
    "Rubber_Trans_Yellow": Rubber_Trans_Yellow,
    "Rubber_Trans_Clear": Rubber_Trans_Clear,
    "Very_Light_Orange": Very_Light_Orange,
    "Bright_Reddish_Lilac": Bright_Reddish_Lilac,
    "Reddish_Brown": Reddish_Brown,
    "Light_Bluish_Grey": Light_Bluish_Grey,
    "Dark_Bluish_Grey": Dark_Bluish_Grey,
    "Medium_Blue": Medium_Blue,
    "Medium_Green": Medium_Green,
    "Speckle_Black_Copper": Speckle_Black_Copper,
    "Speckle_Dark_Bluish_Grey_Silver": Speckle_Dark_Bluish_Grey_Silver,
    "Light_Pink": Light_Pink,
    "Light_Flesh": Light_Flesh,
    "Milky_White": Milky_White,
    "Metallic_Silver": Metallic_Silver,
    "Metallic_Green": Metallic_Green,
    "Metallic_Gold": Metallic_Gold,
    "Metallic_Black": Metallic_Black,
    "Medium_Dark_Flesh": Medium_Dark_Flesh,
    "Medium_Lilac": Medium_Lilac,
    "Dark_Flesh": Dark_Flesh,
    "Metallic_Dark_Grey": Metallic_Dark_Grey,
    "Blue_Violet": Blue_Violet,
    "Flesh": Flesh,
    "Light_Salmon": Light_Salmon,
    "Violet": Violet,
    "Medium_Violet": Medium_Violet,
    "Glitter_Trans_Dark_Pink": Glitter_Trans_Dark_Pink,
    "Medium_Lime": Medium_Lime,
    "Glitter_Trans_Clear": Glitter_Trans_Clear,
    "Aqua": Aqua,
    "Light_Lime": Light_Lime,
    "Light_Orange": Light_Orange,
    "Dark_Nougat": Dark_Nougat,
    "Glitter_Trans_Purple": Glitter_Trans_Purple,
    "Speckle_Black_Silver": Speckle_Black_Silver,
    "Speckle_Black_Gold": Speckle_Black_Gold,
    "Copper": Copper,
    "Pearl_Light_Grey": Pearl_Light_Grey,
    "Metal_Blue": Metal_Blue,
    "Pearl_Light_Gold": Pearl_Light_Gold,
    "Pearl_Dark_Grey": Pearl_Dark_Grey,
    "Pearl_Very_Light_Grey": Pearl_Very_Light_Grey,
    "Very_Light_Bluish_Grey": Very_Light_Bluish_Grey,
    "Flat_Dark_Gold": Flat_Dark_Gold,
    "Flat_Silver": Flat_Silver,
    "Pearl_White": Pearl_White,
    "Metallic_Bright_Red": Metallic_Bright_Red,
    "Metallic_Dark_Green": Metallic_Dark_Green,
    "Reddish_Gold": Reddish_Gold,
    "Bright_Light_Orange": Bright_Light_Orange,
    "Bright_Light_Blue": Bright_Light_Blue,
    "Rust": Rust,
    "Reddish_Lilac": Reddish_Lilac,
    "Lilac": Lilac,
    "Bright_Light_Yellow": Bright_Light_Yellow,
    "Trans_Bright_Light_Orange": Trans_Bright_Light_Orange,
    "Sky_Blue": Sky_Blue,
    "Trans_Fire_Yellow": Trans_Fire_Yellow,
    "Rubber_Black": Rubber_Black,
    "Dark_Blue": Dark_Blue,
    "Rubber_Blue": Rubber_Blue,
    "Trans_Reddish_Lilac": Trans_Reddish_Lilac,
    "Trans_Light_Green": Trans_Light_Green,
    "Dark_Green": Dark_Green,
    "Trans_Light_Blue_Violet": Trans_Light_Blue_Violet,
    "Glow_In_Dark_Trans": Glow_In_Dark_Trans,
    "Flamingo_Pink": Flamingo_Pink,
    "Pearl_Gold": Pearl_Gold,
    "Metallic_Copper": Metallic_Copper,
    "Glitter_Trans_Light_Blue": Glitter_Trans_Light_Blue,
    "Dark_Brown": Dark_Brown,
    "Maersk_Blue": Maersk_Blue,
    "Dark_Red": Dark_Red,
    "Dark_Azure": Dark_Azure,
    "Medium_Azure": Medium_Azure,
    "Light_Aqua": Light_Aqua,
    "Rubber_Red": Rubber_Red,
    "Yellowish_Green": Yellowish_Green,
    "Glow_In_Dark_White": Glow_In_Dark_White,
    "Olive_Green": Olive_Green,
    "Chrome_Gold": Chrome_Gold,
    "Sand_Red": Sand_Red,
    "Glitter_Trans_Neon_Green": Glitter_Trans_Neon_Green,
    "Rubber_Orange": Rubber_Orange,
    "Medium_Dark_Pink": Medium_Dark_Pink,
    "Earth_Orange": Earth_Orange,
    "Sand_Purple": Sand_Purple,
    "Rubber_Light_Grey": Rubber_Light_Grey,
    "Sand_Green": Sand_Green,
    "Sand_Blue": Sand_Blue,
    "Chrome_Silver": Chrome_Silver,
    "Rubber_Dark_Blue": Rubber_Dark_Blue,
    "Rubber_Purple": Rubber_Purple,
    "Fabuland_Brown": Fabuland_Brown,
    "Medium_Orange": Medium_Orange,
    "Dark_Orange": Dark_Orange,
    "Rubber_Lime": Rubber_Lime,
    "Magnet": Magnet,
    "Electric_Contact_Alloy": Electric_Contact_Alloy,
    "Electric_Contact_Copper": Electric_Contact_Copper,
    "Rubber_Light_Bluish_Grey": Rubber_Light_Bluish_Grey,
    "Very_Light_Grey": Very_Light_Grey,
    "Rubber_Flat_Silver": Rubber_Flat_Silver,
    "Rubber_White": Rubber_White,
}



# coding=utf-8
"""
library/parts.py - Auto-generated Part classes for the Python ldraw package.

Copyright (C) 2018 Matthieu Berthomé <rienafairefr@gmail.com>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


<< This file is auto-generated, you shouldn't have to modify it>>
"""
# pylint: disable=invalid-name, line-too-long, too-many-lines

Cook_SHat = "3898"
HatBeanie = "90541"
HatBeret = "90386"
HatBicorne = "2528"
HatBicorneWithEvilSkullAndCrossbonesPattern = "2528p30"
HatBowler = "95674"
HatBowlerWithBlackQuestionMarkPattern = "95674p02"
HatBowlerWithRedFlowerWithYellowCentrePattern = "95674p01"
HatConicalAsian = "93059"
HatConicalAsianWithCurvedSides = "26007"
HatCowboy = "3629"
HatCowboyWithCavalryLogoPattern = "3629pw1"
HatCowboyWithSilverStarPattern = "3629pw2"
HatCrown = "71015"
HatElfWithPointyEars = "13787"
HatElfWithPointyEarsWithBrightGreenTopPattern = "13787p01"
HatFedora = "61506"
HatFez = "85975"
HatHighConeShaped = "2338"
HatImperialGuardShako = "2545"
HatImperialGuardShakoWithRedPlumeAndGoldBadgePattern = "2545p01"
HatKepi = "30135"
HatKnitCap = "41334"
HatPithHelmet = "30172"
HatPolice = "15530"
HatPoliceWithGoldBadgePattern = "15530p01"
HatRag = "2543"
HatTricorne = "2544"
HatTricorneWithWhitePlume_Complete_ = "2544c01"
HatWideBrimFlat = "30167"
HatWithWideBrimDown = "13788"
PoliceHat = "3624"
TopHat = "3878"
WizardsHat = "6131"



#!/usr/bin/env python

"""
ldr2pov.py - An LDraw to POV-Ray convertor tool.

Copyright (C) 2009 David Boddie <david@boddie.org.uk>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import argparse
import sys

from ldraw.tools import vector_position, get_model
from ldraw.writers.povray import POVRayWriter

SKY_SPHERE_FORMAT_STRING = """sky_sphere {
  pigment
  {
    rgb <%s>
  }
}
"""

CAMERA_FORMAT_STRING = """camera {
  location <%s>
  look_at <%s>
}
"""

LIGHT_FORMAT_STRING = """light_source {
  <%1.3f, %1.3f, %1.3f>, rgb <1.0, 1.0, 1.0>
}

light_source {
  <%1.3f, %1.3f, %1.3f>, rgb <1.0, 1.0, 1.0>
}

light_source {
  <%1.3f, %1.3f, %1.3f>, rgb <1.0, 1.0, 1.0>
}

light_source {
  <%1.3f, %1.3f, %1.3f>, rgb <1.0, 1.0, 1.0>
}

"""


def main():
    """ ldr2pov main function """

    description = """Converts the LDraw file to a POV-Ray file.
    
The camera position is a single x,y,z argument where each coordinate
should be specified as a floating point number.
The look at position is a single x,y,z argument where each coordinate
should be specified as a floating point number.
The optional sky colour is a single red,green,blue argument where
each component should be specified as a floating point number between
0.0 and 1.0 inclusive.

"""
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('ldraw_file')
    parser.add_argument('pov_file')
    parser.add_argument('camera_position', type=vector_position)
    parser.add_argument('look_at_position', type=vector_position, required=False,
                        default=vector_position("0,0,0"))
    parser.add_argument('--sky')

    args = parser.parse_args()

    ldr2pov(args.ldraw_file, args.pov_file,
            args.camera_position, args.look_at_position, args.sky)


def ldr2pov(ldraw_path, pov_path,
            camera_position, look_at_position, sky):
    """ actual ldr2pov implementation """
    model, parts = get_model(ldraw_path)

    with open(pov_path, "w") as pov_file:
        pov_file.write('#include "colors.inc"\n\n')
        writer = POVRayWriter(parts, pov_file)
        writer.write(model)

        if not writer.lights:
            lights = (writer.minimum.x - 50.0, writer.maximum.y + 100.0, writer.minimum.z - 50.0,
                      writer.maximum.x + 50.0, writer.maximum.y + 100.0, writer.minimum.z - 50.0,
                      writer.minimum.x - 50.0, writer.maximum.y + 100.0, writer.maximum.z + 50.0,
                      writer.maximum.x + 50.0, writer.maximum.y + 100.0, writer.maximum.z + 50.0)
            pov_file.write(LIGHT_FORMAT_STRING % lights)

        pov_file.write(
            CAMERA_FORMAT_STRING % (camera_position.repr,
                                    look_at_position.repr)
        )

        if sky:
            pov_file.write(SKY_SPHERE_FORMAT_STRING % sky)

    for message, part in writer.warnings:
        sys.stderr.write((message + "\n") % part)


if __name__ == "__main__":
    main()



# coding=utf-8
"""
library/parts.py - Auto-generated Part classes for the Python ldraw package.

Copyright (C) 2018 Matthieu Berthomé <rienafairefr@gmail.com>

This file is part of the ldraw Python package.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


<< This file is auto-generated, you shouldn't have to modify it>>
"""
# pylint: disable=invalid-name, line-too-long, too-many-lines

ArmShortCurvedWithHand = "u9332"
Hand = "3820"
HandFan = "93553"
HandGorillaFist = "11092"
HandHook = "2531"
HandTruck = "2495"
HandTruck_Complete_ = "2495c01"
HelmetCastleUruk_HaiWithLateralComb_HandPattern = "10051p01"
SackWithHandGrab = "10169"
ShieldBroadW_SpikedBottom_CutoutCorner_HandPat_ = "10049p01"
WeaponHandDagger = "88812"
